@@ -57,6 +57,7 @@ DEFINE_EVENT_TYPE(wxEVT_COMMAND_SYMBOL_TREE_DELETE_PROJECT)
 const wxEventType wxEVT_PARSE_THREAD_UPDATED_FILE_SYMBOLS = XRCID("parse_thread_updated_symbols");
 const wxEventType wxEVT_PARSE_THREAD_MESSAGE              = XRCID("parse_thread_update_status_bar");
 const wxEventType wxEVT_PARSE_THREAD_SCAN_INCLUDES_DONE   = XRCID("parse_thread_scan_includes_done");
+const wxEventType wxEVT_PARSE_THREAD_CLEAR_TAGS_CACHE     = XRCID("parse_thread_clear_tags_cache");
 
 
 ParseThread::ParseThread()
@@ -169,16 +170,20 @@ void ParseThread::ProcessIncludes(ParseRequest* req)
 	// Remove from this list all files which starts with one of the crawler search paths
 	wxArrayString searchPaths, excludePaths, filteredFileList;
 	GetSearchPaths( searchPaths, excludePaths );
-
+	
+	DEBUG_MESSAGE( wxString::Format(wxT("Initial workspace files count is %d"), req->_workspaceFiles.size()) ) ;
+	
 	for(size_t i=0; i<req->_workspaceFiles.size(); i++) {
 		wxString name(req->_workspaceFiles.at(i).c_str(), wxConvUTF8);
-		wxFileName fn = name;
+		wxFileName fn(name);
 		fn.MakeAbsolute();
 
 		bool isFromSearchPath(false);
 		for(size_t j=0; j<searchPaths.GetCount(); j++) {
-			wxFileName p ( searchPaths.Item(j) );
+			wxFileName p ( searchPaths.Item(j) + wxFileName::GetPathSeparator());
+			//DEBUG_MESSAGE( wxString::Format(wxT("Comparing %s vs %s"), fn.GetFullPath().c_str(), p.GetPath().c_str()));
 			if( fn.GetFullPath().StartsWith(p.GetPath()) ) {
+				//DEBUG_MESSAGE(wxT("Match was found for file, this file will NOT be parsed!"));
 				isFromSearchPath = true;
 				break;
 			}
@@ -195,34 +200,35 @@ void ParseThread::ProcessIncludes(ParseRequest* req)
 	wxArrayString arrFiles;
 
 	// Clear the results once
-	fcFileOpener::Instance()->ClearResults();
-	fcFileOpener::Instance()->ClearSearchPath();
+	{
+		wxCriticalSectionLocker locker( TagsManagerST::Get()->m_crawlerLocker );
+		
+		fcFileOpener::Instance()->ClearResults();
+		fcFileOpener::Instance()->ClearSearchPath();
 
-	for(size_t i=0; i<searchPaths.GetCount(); i++) {
-		const wxCharBuffer path = _C(searchPaths.Item(i));
-		DEBUG_MESSAGE( wxString::Format(wxT("ParseThread: Using Search Path: %s "), searchPaths.Item(i).c_str()) );
-		fcFileOpener::Instance()->AddSearchPath(path.data());
-	}
+		for(size_t i=0; i<searchPaths.GetCount(); i++) {
+			const wxCharBuffer path = _C(searchPaths.Item(i));
+			DEBUG_MESSAGE( wxString::Format(wxT("ParseThread: Using Search Path: %s "), searchPaths.Item(i).c_str()) );
+			fcFileOpener::Instance()->AddSearchPath(path.data());
+		}
 
-	for(size_t i=0; i<excludePaths.GetCount(); i++) {
-		const wxCharBuffer path = _C(excludePaths.Item(i));
-		DEBUG_MESSAGE( wxString::Format(wxT("ParseThread: Using Exclude Path: %s "), excludePaths.Item(i).c_str()) );
-		fcFileOpener::Instance()->AddExcludePath(path.data());
-	}
+		for(size_t i=0; i<excludePaths.GetCount(); i++) {
+			const wxCharBuffer path = _C(excludePaths.Item(i));
+			DEBUG_MESSAGE( wxString::Format(wxT("ParseThread: Using Exclude Path: %s "), excludePaths.Item(i).c_str()) );
+			fcFileOpener::Instance()->AddExcludePath(path.data());
+		}
 
-	// Before using the 'crawlerScan' we lock it, since it is not mt-safe
-	TagsManagerST::Get()->CrawlerLock();
-	for(size_t i=0; i<filteredFileList.GetCount(); i++) {
-		const wxCharBuffer cfile = filteredFileList.Item(i).mb_str(wxConvUTF8);
-		crawlerScan(cfile.data());
-		if( TestDestroy() ) {
-			TagsManagerST::Get()->CrawlerUnlock();
-			DEBUG_MESSAGE( wxString::Format(wxT("ParseThread::ProcessIncludes -> received 'TestDestroy()'") ) );
-			return;
+		// Before using the 'crawlerScan' we lock it, since it is not mt-safe
+		for(size_t i=0; i<filteredFileList.GetCount(); i++) {
+			const wxCharBuffer cfile = filteredFileList.Item(i).mb_str(wxConvUTF8);
+			crawlerScan(cfile.data());
+			if( TestDestroy() ) {
+				DEBUG_MESSAGE( wxString::Format(wxT("ParseThread::ProcessIncludes -> received 'TestDestroy()'") ) );
+				return;
+			}
 		}
 	}
-
-	TagsManagerST::Get()->CrawlerUnlock();
+	
 	std::set<std::string> *newSet = new std::set<std::string>(fcFileOpener::Instance()->GetResults());
 
 #ifdef PARSE_THREAD_DBG
@@ -331,7 +337,12 @@ void ParseThread::ProcessSimple(ParseRequest* req)
 	// If there is no event handler set to handle this comaprison
 	// results, then nothing more to be done
 	if (m_notifiedWindow ) {
-
+		
+		bool sendClearCacheEvent(false);
+		std::vector<std::pair<wxString, TagEntry> >  realModifiedItems;
+		
+		sendClearCacheEvent = (!deletedItems.empty() || !realModifiedItems.empty() || !newItems.empty());
+		
 		// send "end" event
 		wxCommandEvent e(wxEVT_PARSE_THREAD_UPDATED_FILE_SYMBOLS);
 		wxPostEvent(m_notifiedWindow, e);
@@ -344,7 +355,7 @@ void ParseThread::ProcessSimple(ParseRequest* req)
 			SendEvent(wxEVT_COMMAND_SYMBOL_TREE_ADD_ITEM, req->getFile(), goodNewItems);
 
 		if ( !modifiedItems.empty() ) {
-			std::vector<std::pair<wxString, TagEntry> >  realModifiedItems;
+			
 			for (size_t i=0; i<modifiedItems.size(); i++) {
 				std::pair<wxString, TagEntry> p = modifiedItems.at(i);
 				if (!p.second.GetDifferOnByLineNumber()) {
@@ -355,6 +366,11 @@ void ParseThread::ProcessSimple(ParseRequest* req)
 				SendEvent(wxEVT_COMMAND_SYMBOL_TREE_UPDATE_ITEM, req->getFile(), realModifiedItems);
 			}
 		}
+		
+		if(sendClearCacheEvent) {
+			wxCommandEvent clearCacheEvent(wxEVT_PARSE_THREAD_CLEAR_TAGS_CACHE);
+			wxPostEvent(m_notifiedWindow, clearCacheEvent);
+		}
 	}
 
 	// close the database
@@ -367,27 +383,31 @@ void ParseThread::GetFileListToParse(const wxString& filename, wxArrayString& ar
 	if ( !this->IsCrawlerEnabled() ) {
 		return;
 	}
+	
+	
+	{
+		wxCriticalSectionLocker locker( TagsManagerST::Get()->m_crawlerLocker );
+		
+		wxArrayString includePaths, excludePaths;
+		GetSearchPaths( includePaths, excludePaths );
 
-	wxArrayString includePaths, excludePaths;
-	GetSearchPaths( includePaths, excludePaths );
+		fcFileOpener::Instance()->ClearSearchPath();
+		for(size_t i=0; i<includePaths.GetCount(); i++) {
+			fcFileOpener::Instance()->AddSearchPath( includePaths.Item(i).mb_str(wxConvUTF8).data() );
+		}
 
-	fcFileOpener::Instance()->ClearSearchPath();
-	for(size_t i=0; i<includePaths.GetCount(); i++) {
-		fcFileOpener::Instance()->AddSearchPath( includePaths.Item(i).mb_str(wxConvUTF8).data() );
+		for(size_t i=0; i<excludePaths.GetCount(); i++) {
+			fcFileOpener::Instance()->AddExcludePath(excludePaths.Item(i).mb_str(wxConvUTF8).data());
+		}
+
+		// Invoke the crawler
+		const wxCharBuffer cfile = filename.mb_str(wxConvUTF8);
+
+		// Before using the 'crawlerScan' we lock it, since it is not mt-safe
+		crawlerScan( cfile.data() );
+		
 	}
-
-	for(size_t i=0; i<excludePaths.GetCount(); i++) {
-		fcFileOpener::Instance()->AddExcludePath(excludePaths.Item(i).mb_str(wxConvUTF8).data());
-	}
-
-	// Invoke the crawler
-	const wxCharBuffer cfile = filename.mb_str(wxConvUTF8);
-
-	// Before using the 'crawlerScan' we lock it, since it is not mt-safe
-	TagsManagerST::Get()->CrawlerLock();
-	crawlerScan( cfile.data() );
-	TagsManagerST::Get()->CrawlerUnlock();
-
+	
 	std::set<std::string> fileSet = fcFileOpener::Instance()->GetResults();
 	std::set<std::string>::iterator iter = fileSet.begin();
 	for (; iter != fileSet.end(); iter++ ) {
@@ -430,6 +450,13 @@ void ParseThread::ParseAndStoreFiles(const wxArrayString& arrFiles, int initalCo
 
 		e.SetClientData(new wxString(message.c_str()));
 		m_notifiedWindow->AddPendingEvent( e );
+		
+		// if we added new symbols to the database, send an even to the main thread
+		// to clear the tags cache
+		if(totalSymbols) {
+			wxCommandEvent clearCacheEvent(wxEVT_PARSE_THREAD_CLEAR_TAGS_CACHE);
+			m_notifiedWindow->AddPendingEvent(clearCacheEvent);
+		}
 	}
 }
 
