@@ -26,6 +26,15 @@
 #include <wx/longlong.h>
 #include "tags_storage_sqlite3.h"
 
+#if 0
+#define SQL_LOG 1
+#ifdef __WXMSW__
+# define SQL_LOG_NAME "codelite_tags_sql.log"
+#else
+# define SQL_LOG_NAME "/tmp/codelite_tags_sql.log"
+#endif
+static FILE* log_fp = NULL;
+#endif
 
 //-------------------------------------------------
 // Tags database class implementation
@@ -34,6 +43,7 @@ TagsStorageSQLite::TagsStorageSQLite()
 		: ITagsStorage()
 {
 	m_db    = new wxSQLite3Database();
+	SetUseCache(true);
 }
 
 TagsStorageSQLite::~TagsStorageSQLite()
@@ -95,10 +105,7 @@ void TagsStorageSQLite::CreateSchema()
 		sql = wxT("PRAGMA temp_store = MEMORY;");
 		m_db->ExecuteUpdate(sql);
 
-//		sql = wxT("PRAGMA default_cache_size = 2000;");
-//		m_db->ExecuteUpdate(sql);
-
-		sql = wxT("create  table if not exists tags (ID INTEGER PRIMARY KEY AUTOINCREMENT, name string, file string, line integer, kind string, access string, signature string, pattern string, parent string, inherits string, path string, typeref string, scope string);");
+		sql = wxT("create  table if not exists tags (ID INTEGER PRIMARY KEY AUTOINCREMENT, name string, file string, line integer, kind string, access string, signature string, pattern string, parent string, inherits string, path string, typeref string, scope string, return_value string);");
 		m_db->ExecuteUpdate(sql);
 
 		sql = wxT("create  table if not exists FILES (ID INTEGER PRIMARY KEY AUTOINCREMENT, file string, last_retagged integer);");
@@ -455,24 +462,45 @@ void TagsStorageSQLite::DeleteFromFilesByPrefix(const wxFileName& dbpath, const
 TagEntry* TagsStorageSQLite::FromSQLite3ResultSet(wxSQLite3ResultSet& rs)
 {
 	TagEntry *entry = new TagEntry();
-	entry->SetId       (rs.GetInt(0)   );
-	entry->SetName     (rs.GetString(1));
-	entry->SetFile     (rs.GetString(2));
-	entry->SetLine     (rs.GetInt(3)   );
-	entry->SetKind     (rs.GetString(4));
-	entry->SetAccess   (rs.GetString(5));
-	entry->SetSignature(rs.GetString(6));
-	entry->SetPattern  (rs.GetString(7));
-	entry->SetParent   (rs.GetString(8));
-	entry->SetInherits (rs.GetString(9));
-	entry->SetPath     (rs.GetString(10));
-	entry->SetTyperef  (rs.GetString(11));
-	entry->SetScope    (rs.GetString(12));
+	entry->SetId         (rs.GetInt(0)   );
+	entry->SetName       (rs.GetString(1));
+	entry->SetFile       (rs.GetString(2));
+	entry->SetLine       (rs.GetInt(3)   );
+	entry->SetKind       (rs.GetString(4));
+	entry->SetAccess     (rs.GetString(5));
+	entry->SetSignature  (rs.GetString(6));
+	entry->SetPattern    (rs.GetString(7));
+	entry->SetParent     (rs.GetString(8));
+	entry->SetInherits   (rs.GetString(9));
+	entry->SetPath       (rs.GetString(10));
+	entry->SetTyperef    (rs.GetString(11));
+	entry->SetScope      (rs.GetString(12));
+	entry->SetReturnValue(rs.GetString(13));
 	return entry;
 }
 
 void TagsStorageSQLite::DoFetchTags(const wxString& sql, std::vector<TagEntryPtr>& tags)
 {
+#if SQL_LOG
+	if (!log_fp)
+		log_fp = fopen(SQL_LOG_NAME, "w+b");
+#endif
+
+	if (GetUseCache()) {
+		if (m_cache.Get(sql, tags) == true) {
+#if SQL_LOG
+			fprintf(log_fp, "[CACHED ITEMS] %s\n", sql.mb_str(wxConvUTF8).data());
+			fflush(log_fp);
+#endif
+			return;
+		}
+	}
+
+#if SQL_LOG
+	fprintf(log_fp, "%s\n", sql.mb_str(wxConvUTF8).data());
+	fflush(log_fp);
+#endif
+
 	// try the cache first
 	try {
 		wxSQLite3ResultSet ex_rs;
@@ -489,10 +517,34 @@ void TagsStorageSQLite::DoFetchTags(const wxString& sql, std::vector<TagEntryPtr
 	} catch (wxSQLite3Exception &e) {
 		wxUnusedVar ( e );
 	}
+
+	if (GetUseCache()) {
+		m_cache.Store(sql, tags);
+	}
 }
 
 void TagsStorageSQLite::DoFetchTags(const wxString& sql, std::vector<TagEntryPtr>& tags, const wxArrayString& kinds)
 {
+#if SQL_LOG
+	if (!log_fp)
+		log_fp = fopen(SQL_LOG_NAME, "w+b");
+#endif
+
+	if (GetUseCache()) {
+		if (m_cache.Get(sql, kinds, tags) == true) {
+#if SQL_LOG
+			fprintf(log_fp, "[CACHED ITEMS] %s\n", sql.mb_str(wxConvUTF8).data());
+			fflush(log_fp);
+#endif
+			return;
+		}
+	}
+
+#if SQL_LOG
+	fprintf(log_fp, "%s\n", sql.mb_str(wxConvUTF8).data());
+	fflush(log_fp);
+#endif
+
 	try {
 		wxSQLite3ResultSet ex_rs;
 		ex_rs = Query(sql);
@@ -514,6 +566,10 @@ void TagsStorageSQLite::DoFetchTags(const wxString& sql, std::vector<TagEntryPtr
 	} catch (wxSQLite3Exception &e) {
 		wxUnusedVar ( e );
 	}
+
+	if (GetUseCache()) {
+		m_cache.Store(sql, kinds, tags);
+	}
 }
 
 void TagsStorageSQLite::GetTagsByScopeAndName(const wxString& scope, const wxString& name, bool partialNameAllowed, std::vector<TagEntryPtr> &tags)
@@ -745,8 +801,12 @@ int TagsStorageSQLite::InsertTagEntry(const TagEntry& tag)
 	if ( !tag.IsOk() )
 		return TagOk;
 
+	if (GetUseCache()) {
+		ClearCache();
+	}
+
 	try {
-		wxSQLite3Statement statement = m_db->PrepareStatement(wxT("INSERT INTO TAGS VALUES (NULL, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"));
+		wxSQLite3Statement statement = m_db->PrepareStatement(wxT("INSERT INTO TAGS VALUES (NULL, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"));
 		statement.Bind(1,  tag.GetName());
 		statement.Bind(2,  tag.GetFile());
 		statement.Bind(3,  tag.GetLine());
@@ -759,6 +819,7 @@ int TagsStorageSQLite::InsertTagEntry(const TagEntry& tag)
 		statement.Bind(10, tag.GetPath());
 		statement.Bind(11, tag.GetTyperef());
 		statement.Bind(12, tag.GetScope());
+		statement.Bind(13, tag.GetReturnValue());
 		statement.ExecuteUpdate();
 	} catch (wxSQLite3Exception& exc) {
 
@@ -777,7 +838,7 @@ int TagsStorageSQLite::UpdateTagEntry(const TagEntry& tag)
 		return TagOk;
 
 	try {
-		wxSQLite3Statement statement = m_db->PrepareStatement(wxT("UPDATE TAGS SET Name=?, File=?, Line=?, Access=?, Pattern=?, Parent=?, Inherits=?, Typeref=?, Scope=? WHERE Kind=? AND Signature=? AND Path=?"));
+		wxSQLite3Statement statement = m_db->PrepareStatement(wxT("UPDATE TAGS SET Name=?, File=?, Line=?, Access=?, Pattern=?, Parent=?, Inherits=?, Typeref=?, Scope=?, Return_Value=? WHERE Kind=? AND Signature=? AND Path=?"));
 		// update
 		statement.Bind(1,  tag.GetName());
 		statement.Bind(2,  tag.GetFile());
@@ -788,11 +849,12 @@ int TagsStorageSQLite::UpdateTagEntry(const TagEntry& tag)
 		statement.Bind(7,  tag.GetInherits());
 		statement.Bind(8,  tag.GetTyperef());
 		statement.Bind(9,  tag.GetScope());
+		statement.Bind(10, tag.GetReturnValue());
 
 		// where?
-		statement.Bind(10, tag.GetKind());
-		statement.Bind(11, tag.GetSignature());
-		statement.Bind(12, tag.GetPath());
+		statement.Bind(11, tag.GetKind());
+		statement.Bind(12, tag.GetSignature());
+		statement.Bind(13, tag.GetPath());
 
 		statement.ExecuteUpdate();
 	} catch (wxSQLite3Exception& exc) {
@@ -803,10 +865,30 @@ int TagsStorageSQLite::UpdateTagEntry(const TagEntry& tag)
 	return TagOk;
 }
 
-bool TagsStorageSQLite::IsTypeAndScopeContainer(const wxString& typeName, wxString& scope)
+bool TagsStorageSQLite::IsTypeAndScopeContainer(wxString& typeName, wxString& scope)
 {
 	wxString sql;
-	sql << wxT("select scope,kind from tags where name='") << typeName << wxT("'");
+
+	// Break the typename to 'name' and scope
+	wxString typeNameNoScope(typeName.AfterLast(wxT(':')));
+	wxString scopeOne       (typeName.BeforeLast(wxT(':')));
+
+	if (scopeOne.EndsWith(wxT(":")))
+		scopeOne.RemoveLast();
+
+	wxString combinedScope;
+
+	if (scope != wxT("<global>"))
+		combinedScope << scope;
+
+	if (scopeOne.IsEmpty() == false) {
+		if (combinedScope.IsEmpty() == false)
+			combinedScope << wxT("::");
+		combinedScope << scopeOne;
+	}
+
+
+	sql << wxT("select scope,kind from tags where name='") << typeNameNoScope << wxT("'");
 
 	bool found_global(false);
 
@@ -817,9 +899,25 @@ bool TagsStorageSQLite::IsTypeAndScopeContainer(const wxString& typeName, wxStri
 			wxString kindFounded  (rs.GetString(1));
 
 			bool containerKind = kindFounded == wxT("struct") || kindFounded == wxT("class");
-			if (scopeFounded == scope && containerKind) {
+			if (scopeFounded == combinedScope && containerKind) {
+				scope    = combinedScope;
+				typeName = typeNameNoScope;
 				//we got an exact match
 				return true;
+
+			} else if (scopeFounded == scopeOne && containerKind) {
+				// this is equal to cases like this:
+				// class A {
+				// typedef std::list<int> List;
+				// List l;
+				// };
+				// the combinedScope will be: 'A::std'
+				// however, the actual scope is 'std'
+				scope    = scopeOne;
+				typeName = typeNameNoScope;
+				//we got an exact match
+				return true;
+
 			} else if ( containerKind && scopeFounded == wxT("<global>") ) {
 				found_global = true;
 			}
@@ -831,27 +929,61 @@ bool TagsStorageSQLite::IsTypeAndScopeContainer(const wxString& typeName, wxStri
 
 	// if we reached here, it means we did not find any exact match
 	if ( found_global ) {
-		scope = wxT("<global>");
+		scope    = wxT("<global>");
+		typeName = typeNameNoScope;
 		return true;
 	}
+
 	return false;
 }
 
 
-bool TagsStorageSQLite::IsTypeAndScopeExist(const wxString& typeName, wxString& scope)
+bool TagsStorageSQLite::IsTypeAndScopeExist(wxString& typeName, wxString& scope)
 {
 	wxString sql;
-	sql << wxT("select scope from tags where name='") << typeName << wxT("'");
-	bool found_global(false);
+	wxString strippedName;
+	wxString secondScope;
+	wxString bestScope;
+	wxString parent;
+	wxString tmpScope(scope);
+
+	strippedName = typeName.AfterLast(wxT(':'));
+	secondScope  = typeName.BeforeLast(wxT(':'));
+
+	if (secondScope.EndsWith(wxT(":")))
+		secondScope.RemoveLast();
+
+	if (strippedName.IsEmpty())
+		return false;
+
+	sql << wxT("select scope,parent from tags where name='") << strippedName << wxT("' and kind in ('class', 'struct', 'typedef') LIMIT 50");
+	int     foundOther(0);
+	wxString scopeFounded;
+	wxString parentFounded;
+
+	if (secondScope.IsEmpty() == false)
+		tmpScope << wxT("::") << secondScope;
+
+	parent = tmpScope.AfterLast(wxT(':'));
+
 	try {
 		wxSQLite3ResultSet rs = Query(sql);
 		while (rs.NextRow()) {
-			wxString scopeFounded (rs.GetString(0));
-			if ( scopeFounded == scope ) {
+
+			scopeFounded  = rs.GetString(0);
+			parentFounded = rs.GetString(1);
+
+			if ( scopeFounded == tmpScope ) {
 				// exact match
+				scope    = scopeFounded;
+				typeName = strippedName;
 				return true;
-			} else if ( scopeFounded == wxT("<global>") ) {
-				found_global = true;
+
+			} else if (parentFounded == parent) {
+				bestScope  = scopeFounded;
+
+			} else {
+				foundOther++;
 			}
 		}
 
@@ -860,8 +992,14 @@ bool TagsStorageSQLite::IsTypeAndScopeExist(const wxString& typeName, wxString&
 	}
 
 	// if we reached here, it means we did not find any exact match
-	if ( found_global ) {
-		scope = wxT("<global>");
+	if ( bestScope.IsEmpty() == false ) {
+		scope    = bestScope;
+		typeName = strippedName;
+		return true;
+
+	} else if ( foundOther == 1) {
+		scope    = scopeFounded;
+		typeName = strippedName;
 		return true;
 	}
 	return false;
@@ -974,7 +1112,7 @@ void TagsStorageSQLite::GetTagsByPath(const wxString& path, std::vector<TagEntry
 	if (path.empty()) return;
 
 	wxString sql;
-	sql << wxT("select * from tags where path ='") << path << wxT("'");
+	sql << wxT("select * from tags where path ='") << path << wxT("' LIMIT 1");
 	DoFetchTags(sql, tags);
 }
 
@@ -1010,3 +1148,218 @@ void TagsStorageSQLite::GetGlobalFunctions(std::vector<TagEntryPtr>& tags)
 	sql << wxT("select * from tags where scope = '<global>' AND kind IN ('function', 'prototype') LIMIT ") << GetSingleSearchLimit();
 	DoFetchTags(sql, tags);
 }
+
+void TagsStorageSQLite::GetTagsByFiles(const wxArrayString& files, std::vector<TagEntryPtr>& tags)
+{
+	if(files.IsEmpty())
+		return;
+		
+	wxString sql;
+	sql << wxT("select * from tags where file in (");
+	for(size_t i=0; i<files.GetCount(); i++) {
+		sql << wxT("'") << files.Item(i) << wxT("',");
+	}
+	sql.RemoveLast();
+	sql << wxT(")");
+	DoFetchTags(sql, tags);
+}
+
+void TagsStorageSQLite::GetTagsByFilesAndScope(const wxArrayString& files, const wxString& scope, std::vector<TagEntryPtr>& tags)
+{
+	if(files.IsEmpty())
+		return;
+		
+	wxString sql;
+	sql << wxT("select * from tags where file in (");
+	for(size_t i=0; i<files.GetCount(); i++) {
+		sql << wxT("'") << files.Item(i) << wxT("',");
+	}
+	sql.RemoveLast();
+	sql << wxT(")");
+	
+	sql << wxT(" AND scope='") << scope << wxT("'");
+	DoFetchTags(sql, tags);
+}
+
+void TagsStorageSQLite::GetTagsByFilesKindAndScope(const wxArrayString& files, const wxArrayString& kinds, const wxString& scope, std::vector<TagEntryPtr>& tags)
+{
+	if(files.IsEmpty())
+		return;
+		
+	wxString sql;
+	sql << wxT("select * from tags where file in (");
+	for(size_t i=0; i<files.GetCount(); i++) {
+		sql << wxT("'") << files.Item(i) << wxT("',");
+	}
+	sql.RemoveLast();
+	sql << wxT(")");
+	
+	sql << wxT(" AND scope='") << scope << wxT("'");
+	DoFetchTags(sql, tags, kinds);
+}
+
+void TagsStorageSQLite::GetTagsByFilesScopeTyperefAndKind(const wxArrayString& files, const wxArrayString& kinds, const wxString& scope, const wxString& typeref, std::vector<TagEntryPtr>& tags)
+{
+	if(files.IsEmpty())
+		return;
+		
+	wxString sql;
+	sql << wxT("select * from tags where file in (");
+	for(size_t i=0; i<files.GetCount(); i++) {
+		sql << wxT("'") << files.Item(i) << wxT("',");
+	}
+	sql.RemoveLast();
+	sql << wxT(")");
+	
+	sql << wxT(" AND scope='") << scope << wxT("'");
+	sql << wxT(" AND typeref='") << typeref << wxT("'");
+	DoFetchTags(sql, tags, kinds);
+}
+
+void TagsStorageSQLite::GetTagsByKindLimit(const wxArrayString& kinds, const wxString& orderingColumn, int order, int limit, const wxString &partName, std::vector<TagEntryPtr>& tags)
+{
+	wxString sql;
+	sql << wxT("select * from tags where kind in (");
+	for (size_t i=0; i<kinds.GetCount(); i++) {
+		sql << wxT("'") << kinds.Item(i) << wxT("',");
+	}
+	sql.RemoveLast();
+	sql << wxT(") ");
+
+	if ( orderingColumn.IsEmpty() == false ) {
+		sql << wxT("order by ") << orderingColumn;
+		switch (order) {
+		case ITagsStorage::OrderAsc:
+			sql << wxT(" ASC");
+			break;
+		case ITagsStorage::OrderDesc:
+			sql << wxT(" DESC");
+			break;
+		case ITagsStorage::OrderNone:
+		default:
+			break;
+		}
+	}
+
+	if (partName.IsEmpty() == false) {
+		wxString tmpName(partName);
+		tmpName.Replace(wxT("_"), wxT("^_"));
+		sql << wxT(" AND name like '%%") << tmpName << wxT("%%' ESCAPE '^' ");
+	}
+
+	if (limit > 0) {
+		sql << wxT(" LIMIT ") << limit;
+	}
+
+	DoFetchTags(sql, tags);
+
+}
+bool TagsStorageSQLite::IsTypeAndScopeExistLimitOne(const wxString& typeName, const wxString& scope)
+{
+	wxString sql;
+	wxString path;
+
+	// Build the path
+	if (scope.IsEmpty() == false && scope != wxT("<global>"))
+		path << scope << wxT("::");
+
+	path << typeName;
+	sql << wxT("select ID from tags where path='") << path << wxT("' and kind in ('class', 'struct', 'typedef') LIMIT 1");
+
+	try {
+		wxSQLite3ResultSet rs = Query(sql);
+		if (rs.NextRow()) {
+			return true;
+		}
+
+	} catch ( wxSQLite3Exception& e) {
+		wxUnusedVar(e);
+	}
+	return false;
+}
+
+//---------------------------------------------------------------------
+//-----------------------------TagsStorageSQLiteCache -----------------
+//---------------------------------------------------------------------
+
+TagsStorageSQLiteCache::TagsStorageSQLiteCache()
+{
+}
+
+TagsStorageSQLiteCache::~TagsStorageSQLiteCache()
+{
+	m_cache.clear();
+}
+
+bool TagsStorageSQLiteCache::Get(const wxString& sql, std::vector<TagEntryPtr>& tags)
+{
+	return DoGet(sql, tags);
+}
+
+bool TagsStorageSQLiteCache::Get(const wxString& sql, const wxArrayString& kind, std::vector<TagEntryPtr>& tags)
+{
+	wxString key;
+	key << sql;
+	for (size_t i=0; i<kind.GetCount(); i++) {
+		key << wxT("@") << kind.Item(i);
+	}
+
+	return DoGet(key, tags);
+}
+
+void TagsStorageSQLiteCache::Store(const wxString& sql, const std::vector<TagEntryPtr>& tags)
+{
+	DoStore(sql, tags);
+}
+
+void TagsStorageSQLiteCache::Clear()
+{
+#if SQL_LOG
+	if (!log_fp)
+		log_fp = fopen(SQL_LOG_NAME, "w+b");
+#endif
+
+#if SQL_LOG
+	fprintf(log_fp, "[CACHE CLEARED]\n");
+	fflush(log_fp);
+#endif
+
+	m_cache.clear();
+}
+
+void TagsStorageSQLiteCache::Store(const wxString& sql, const wxArrayString& kind, const std::vector<TagEntryPtr>& tags)
+{
+	wxString key;
+	key << sql;
+	for (size_t i=0; i<kind.GetCount(); i++) {
+		key << wxT("@") << kind.Item(i);
+	}
+	DoStore(key, tags);
+}
+
+bool TagsStorageSQLiteCache::DoGet(const wxString& key, std::vector<TagEntryPtr>& tags)
+{
+	std::map<wxString, std::vector<TagEntryPtr> >::iterator iter = m_cache.find(key);
+	if (iter != m_cache.end()) {
+		// Append the results to the output tags
+		tags.insert(tags.end(), iter->second.begin(), iter->second.end());
+		return true;
+	}
+	return false;
+}
+
+void TagsStorageSQLiteCache::DoStore(const wxString& key, const std::vector<TagEntryPtr>& tags)
+{
+	m_cache[key] = tags;
+}
+
+void TagsStorageSQLite::ClearCache()
+{
+	m_cache.Clear();
+}
+
+void TagsStorageSQLite::SetUseCache(bool useCache)
+{
+	ITagsStorage::SetUseCache(useCache);
+}
+
