@@ -170,14 +170,15 @@ GizmosPlugin::~GizmosPlugin()
 {
 }
 
-wxToolBar *GizmosPlugin::CreateToolBar(wxWindow *parent)
+clToolBar *GizmosPlugin::CreateToolBar(wxWindow *parent)
 {
-	wxToolBar *tb(NULL);
+	clToolBar *tb(NULL);
 	if (m_mgr->AllowToolbar()) {
 		//support both toolbars icon size
 		int size = m_mgr->GetToolbarIconSize();
 
-		tb = new wxToolBar(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTB_FLAT | wxTB_NODIVIDER);
+
+		tb = new clToolBar(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, clTB_DEFAULT_STYLE);
 		tb->SetToolBitmapSize(wxSize(size, size));
 
 		if (size == 24) {
@@ -185,11 +186,19 @@ wxToolBar *GizmosPlugin::CreateToolBar(wxWindow *parent)
 		} else {
 			tb->AddTool(XRCID("gizmos_options"), wxT("Gizmos..."), wxXmlResource::Get()->LoadBitmap(wxT("plugin16")), wxT("Open Gizmos quick menu"));
 		}
+
+		// When using AUI, make this toolitem a dropdown button
+#if USE_AUI_TOOLBAR
+		tb->SetToolDropDown(XRCID("gizmos_options"), true);
+		m_mgr->GetTheApp()->Connect(XRCID("gizmos_options"), wxEVT_COMMAND_AUITOOLBAR_TOOL_DROPDOWN, wxAuiToolBarEventHandler(GizmosPlugin::OnGizmosAUI), NULL, (wxEvtHandler*)this);
+#endif
 		tb->Realize();
 	}
 
 	//Connect the events to us
+#if !USE_AUI_TOOLBAR
 	m_mgr->GetTheApp()->Connect(XRCID("gizmos_options"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(GizmosPlugin::OnGizmos   ), NULL, (wxEvtHandler*)this);
+#endif
 	m_mgr->GetTheApp()->Connect(XRCID("gizmos_options"), wxEVT_UPDATE_UI,             wxUpdateUIEventHandler(GizmosPlugin::OnGizmosUI), NULL, (wxEvtHandler*)this);
 
 	m_mgr->GetTheApp()->Connect(ID_MI_NEW_CODELITE_PLUGIN, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(GizmosPlugin::OnNewPlugin), NULL, (wxEvtHandler*)this);
@@ -246,6 +255,8 @@ void GizmosPlugin::UnHookPopupMenu(wxMenu *menu, MenuType type)
 
 void GizmosPlugin::UnPlug()
 {
+	m_mgr->GetTheApp()->Disconnect(XRCID("gizmos_options"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(GizmosPlugin::OnGizmos   ), NULL, (wxEvtHandler*)this);
+	m_mgr->GetTheApp()->Disconnect(XRCID("gizmos_options"), wxEVT_UPDATE_UI,             wxUpdateUIEventHandler(GizmosPlugin::OnGizmosUI), NULL, (wxEvtHandler*)this);
 }
 
 void GizmosPlugin::OnNewPlugin(wxCommandEvent &e)
@@ -360,12 +371,14 @@ void GizmosPlugin::DoCreateNewPlugin()
 
 void GizmosPlugin::OnNewPluginUI(wxUpdateUIEvent &e)
 {
+	CHECK_CL_SHUTDOWN();
 	//we enable the button only when workspace is opened
 	e.Enable(m_mgr->IsWorkspaceOpen());
 }
 
 void GizmosPlugin::OnNewClassUI(wxUpdateUIEvent &e)
 {
+	CHECK_CL_SHUTDOWN();
 	//we enable the button only when workspace is opened
 	e.Enable(m_mgr->IsWorkspaceOpen());
 }
@@ -594,6 +607,7 @@ void GizmosPlugin::DoCreateNewWxProject()
 
 void GizmosPlugin::OnNewWxProjectUI(wxUpdateUIEvent &e)
 {
+	CHECK_CL_SHUTDOWN();
 	//we enable the button only when workspace is opened
 	e.Enable(m_mgr->IsWorkspaceOpen());
 }
@@ -876,7 +890,7 @@ wxString GizmosPlugin::DoGetVirtualFuncImpl(const NewClassInfo &info)
 	wxString impl;
 	for (std::vector< TagEntryPtr >::size_type i=0; i< tags.size(); i++) {
 		TagEntryPtr tt = tags.at(i);
-		impl << m_mgr->GetTagsManager()->FormatFunction(tt, true, info.name);
+		impl << m_mgr->GetTagsManager()->FormatFunction(tt, FunctionFormat_Impl, info.name);
 	}
 	return impl;
 }
@@ -935,30 +949,13 @@ void GizmosPlugin::OnGizmos(wxCommandEvent& e)
 {
 	// open a popup menu
 	wxUnusedVar(e);
-
-#ifdef __WXMSW__
-	wxFont font = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
-#endif
-
-	wxMenu popupMenu;
-
-	std::map<wxString, int> options;
-	options[MI_NEW_CODELITE_PLUGIN] = ID_MI_NEW_CODELITE_PLUGIN;
-	options[MI_NEW_NEW_CLASS      ] = ID_MI_NEW_NEW_CLASS;
-	options[MI_NEW_WX_PROJECT     ] = ID_MI_NEW_WX_PROJECT;
-
-	std::map<wxString, int>::iterator iter = options.begin();
-	for (; iter != options.end(); iter++) {
-		int      id   = (*iter).second;
-		wxString text = (*iter).first;
-		wxMenuItem *item = new wxMenuItem(&popupMenu, id, text, text, wxITEM_NORMAL);
-		popupMenu.Append(item);
-	}
-	m_mgr->GetTheApp()->GetTopWindow()->PopupMenu(&popupMenu);
+	wxPoint pt;
+	DoPopupButtonMenu(pt);
 }
 
 void GizmosPlugin::OnGizmosUI(wxUpdateUIEvent& e)
 {
+	CHECK_CL_SHUTDOWN();
 	e.Enable(m_mgr->IsWorkspaceOpen());
 }
 
@@ -992,3 +989,47 @@ void GizmosPlugin::GizmosRemoveDuplicates(std::vector<TagEntryPtr>& src, std::ve
 		target.push_back( iter->second );
 	}
 }
+
+void GizmosPlugin::DoPopupButtonMenu(wxPoint pt)
+{
+#ifdef __WXMSW__
+	wxFont font = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
+#endif
+
+	wxMenu popupMenu;
+
+	std::map<wxString, int> options;
+	options[MI_NEW_CODELITE_PLUGIN] = ID_MI_NEW_CODELITE_PLUGIN;
+	options[MI_NEW_NEW_CLASS      ] = ID_MI_NEW_NEW_CLASS;
+	options[MI_NEW_WX_PROJECT     ] = ID_MI_NEW_WX_PROJECT;
+
+	std::map<wxString, int>::iterator iter = options.begin();
+	for (; iter != options.end(); iter++) {
+		int      id   = (*iter).second;
+		wxString text = (*iter).first;
+		wxMenuItem *item = new wxMenuItem(&popupMenu, id, text, text, wxITEM_NORMAL);
+		popupMenu.Append(item);
+	}
+	m_mgr->GetTheApp()->GetTopWindow()->PopupMenu(&popupMenu, pt);
+}
+
+#if USE_AUI_TOOLBAR
+void GizmosPlugin::OnGizmosAUI(wxAuiToolBarEvent& e)
+{
+    if (e.IsDropDownClicked())
+    {
+        wxAuiToolBar* tb = static_cast<wxAuiToolBar*>(e.GetEventObject());
+        tb->SetToolSticky(e.GetId(), true);
+
+		// line up our menu with the button
+        wxRect rect = tb->GetToolRect(e.GetId());
+        wxPoint pt = tb->ClientToScreen(rect.GetBottomLeft());
+        pt = m_mgr->GetTheApp()->GetTopWindow()->ScreenToClient(pt);
+
+		DoPopupButtonMenu(pt);
+		tb->SetToolSticky(e.GetId(), false);
+	}
+}
+#endif
+
+
