@@ -74,6 +74,14 @@ enum BreakpointOrigin {
 	BO_Other
 };
 
+enum DisplayFormat {
+	DBG_DF_NATURAL = 0,
+	DBG_DF_HEXADECIMAL,
+	DBG_DF_BINARY,
+	DBG_DF_DECIMAL,
+	DBG_DF_OCTAL
+};
+
 //-------------------------------------------------------
 // Data structures used by the debugger
 //-------------------------------------------------------
@@ -175,7 +183,15 @@ public:
 			conditions(BI.conditions),
 			at(BI.at),
 			what(BI.what),
-			origin(BI.origin) {}
+			origin(BI.origin)
+	{
+		// Normalize the file name
+		if(file.IsEmpty() == false) {
+			wxFileName fn(file);
+			fn.Normalize(wxPATH_NORM_ALL & ~wxPATH_NORM_LONG);
+			file = fn.GetFullPath();
+		}
+	}
 
 	BreakpointInfo() : lineno(-1), regex(false), debugger_id(-1), bp_type(BP_type_break),
 			ignore_number(0), is_enabled(true), is_temp(false), watchpoint_type(WP_watch), origin(BO_Other) {}
@@ -189,9 +205,12 @@ public:
 	}
 
 	void Create(wxString filename, int line, int int_id, int ext_id = -1) {
+		wxFileName fn(filename);
+		fn.Normalize(wxPATH_NORM_ALL & ~wxPATH_NORM_LONG);
+
 		bp_type = BP_type_break;
 		lineno = line;
-		file = filename;
+		file = filename.IsEmpty() ? wxT("") : fn.GetFullPath();
 		internal_id = int_id;
 		debugger_id = ext_id;
 	}
@@ -319,7 +338,7 @@ public:
 	}
 };
 
-class DebuggerInformation
+class DebuggerInformation : public SerializedObject
 {
 public:
 	wxString  name;
@@ -336,7 +355,8 @@ public:
 	wxString  startupCommands;
 	int       maxDisplayStringSize;
 	bool      resolveLocals;
-
+	bool      autoExpandTipItems;
+	bool      applyBreakpointsAfterProgramStarted;
 public:
 	DebuggerInformation()
 			: name(wxEmptyString)
@@ -352,8 +372,53 @@ public:
 			, debugAsserts(false)
 			, startupCommands(wxEmptyString)
 			, maxDisplayStringSize(200)
-			, resolveLocals (false) {}
-	~DebuggerInformation() {}
+			, resolveLocals     (false)
+			, autoExpandTipItems(true)
+			, applyBreakpointsAfterProgramStarted(false)
+	{
+	}
+
+	virtual ~DebuggerInformation() {}
+
+	void Serialize(Archive &arch) {
+		arch.Write(wxT("name"),                                name);
+		arch.Write(wxT("path"),                                path);
+		arch.Write(wxT("enableDebugLog"),                      enableDebugLog);
+		arch.Write(wxT("enablePendingBreakpoints"),            enablePendingBreakpoints);
+		arch.Write(wxT("breakAtWinMain"),                      breakAtWinMain);
+		arch.Write(wxT("showTerminal"),                        showTerminal);
+		arch.Write(wxT("consoleCommand"),                      consoleCommand);
+		arch.Write(wxT("useRelativeFilePaths"),                useRelativeFilePaths);
+		arch.Write(wxT("catchThrow"),                          catchThrow);
+		arch.Write(wxT("showTooltips"),                        showTooltips);
+		arch.Write(wxT("debugAsserts"),                        debugAsserts);
+		arch.WriteCData(wxT("startupCommands"),                startupCommands);
+		arch.Write(wxT("maxDisplayStringSize"),                maxDisplayStringSize);
+		arch.Write(wxT("resolveLocals"),                       resolveLocals);
+		arch.Write(wxT("autoExpandTipItems"),                  autoExpandTipItems);
+		arch.Write(wxT("applyBreakpointsAfterProgramStarted"), applyBreakpointsAfterProgramStarted);
+	}
+
+	void DeSerialize(Archive &arch) {
+		arch.Read(wxT("name"),                                name);
+		arch.Read(wxT("path"),                                path);
+		arch.Read(wxT("enableDebugLog"),                      enableDebugLog);
+		arch.Read(wxT("enablePendingBreakpoints"),            enablePendingBreakpoints);
+		arch.Read(wxT("breakAtWinMain"),                      breakAtWinMain);
+		arch.Read(wxT("showTerminal"),                        showTerminal);
+		arch.Read(wxT("consoleCommand"),                      consoleCommand);
+		arch.Read(wxT("useRelativeFilePaths"),                useRelativeFilePaths);
+		arch.Read(wxT("catchThrow"),                          catchThrow);
+		arch.Read(wxT("showTooltips"),                        showTooltips);
+		arch.Read(wxT("debugAsserts"),                        debugAsserts);
+		arch.ReadCData(wxT("startupCommands"),                startupCommands);
+		startupCommands.Trim();
+
+		arch.Read(wxT("maxDisplayStringSize"),                maxDisplayStringSize);
+		arch.Read(wxT("resolveLocals"),                       resolveLocals);
+		arch.Read(wxT("autoExpandTipItems"),                  autoExpandTipItems);
+		arch.Read(wxT("applyBreakpointsAfterProgramStarted"), applyBreakpointsAfterProgramStarted);
+	}
 };
 
 class IDebugger;
@@ -438,7 +503,7 @@ public:
 		m_env = env;
 	}
 
-	void SetDebuggerInformation(const DebuggerInformation& info) {
+	virtual void SetDebuggerInformation(const DebuggerInformation& info) {
 		m_info = info;
 	}
 
@@ -449,11 +514,11 @@ public:
 	void SetIsRemoteDebugging(bool isRemoteDebugging) {
 		this->m_isRemoteDebugging = isRemoteDebugging;
 	}
-	
+
 	bool GetIsRemoteDebugging() const {
 		return m_isRemoteDebugging;
 	}
-	
+
 	/**
 	 * \brief Sets the logging level 'on the fly'
 	 * \param level the new level
@@ -690,7 +755,7 @@ public:
 	 * @brief evaluate variable object
 	 * @param name variable object
 	 */
-	virtual bool EvaluateVariableObject(const wxString &name, int userReason) = 0;
+	virtual bool EvaluateVariableObject(const wxString &name, DisplayFormat displayFormat, int userReason) = 0;
 };
 
 //-----------------------------------------------------------
