@@ -26,13 +26,16 @@
 #ifndef IMANAGER_H
 #define IMANAGER_H
 
+#include "output_pane_book.h"
 #include "ieditor.h"
 #include "iconfigtool.h"
 #include "ikeyboard.h"
 #include "wx/treectrl.h"
-#include "custom_notebook.h"
+#include "notebook_ex.h"
 #include "optionsconfig.h"
 #include "queuecommand.h"
+#include <wx/aui/framemanager.h>
+
 
 class TagsManager;
 class Workspace;
@@ -43,6 +46,8 @@ class IPlugin;
 class BuildManager;
 class BuildSettingsConfig;
 class NavMgr;
+class IMacroManager;
+class wxAuiManager;
 
 //--------------------------
 //Auxulary class
@@ -122,18 +127,21 @@ public:
 	 * @sa TreeType
 	 */
 	virtual wxTreeCtrl *GetTree(TreeType type) = 0;
+
 	/**
 	 * @brief return a pointer to the workspace pane notebook (the one with the 'workspace' title)
 	 * @return pointer to Notebook
 	 * @sa Notebook
 	 */
 	virtual Notebook *GetWorkspacePaneNotebook() = 0;
+
 	/**
 	 * @brief return a pointer to the output pane notebook (the one with the 'output' title)
 	 * @return pointer to Notebook
 	 * @sa Notebook
 	 */
-	virtual Notebook *GetOutputPaneNotebook() = 0;
+	virtual OutputPaneBook* GetOutputPaneNotebook() = 0;
+
 	/**
 	 * @brief return the startup directory of CodeLite which is also the base directory for searching installation files
 	 * @return a full path to the startup directory
@@ -190,7 +198,7 @@ public:
 	 * @param vdFullPath virtual directory full path in the form of <project>:vd1:vd2:...:vdN
 	 * @param paths an array of files to add
 	 * @return true on sucesss, false otherwise
-	 */	
+	 */
 	virtual bool AddFilesToVirtualFolderIntelligently(const wxString& vdFullPath, wxArrayString& paths) = 0;
 
 	/**
@@ -315,6 +323,12 @@ public:
 	 */
 	virtual bool IsBuildInProgress() const = 0;
 
+	/**
+	 * @brief return true if a shutdown is currently in progress
+	 * @return
+	 */
+	virtual bool IsShutdownInProgress() const = 0;
+
 	/**
 	 * return true if the last buid ended successfully
 	 */
@@ -364,13 +378,18 @@ public:
      * @brief select a window in mainbook
      */
     virtual bool SelectPage(wxWindow *win) = 0;
-	
-	
+
+
 	/**
 	 * @brief open new editor "untitiled"
 	 * @return pointer to the editor
 	 */
 	virtual IEditor *NewEditor() = 0;
+
+	/**
+	 * @brief return the macro manager
+	 */
+	virtual IMacroManager *GetMacrosManager() = 0;
 };
 
 
