@@ -25,8 +25,22 @@
 #ifndef PLUGIN_H
 #define PLUGIN_H
 
+// Windows headers define it
+#ifdef __WXMSW__
+#ifdef Yield
+#undef Yield
+#endif
+#endif
+
+#include <wx/app.h>
 #include "imanager.h"
 #include "wx/toolbar.h"
+#include <wx/pen.h>
+#include "cl_aui_tb_are.h"
+#include "cl_defs.h"
+#if USE_AUI_TOOLBAR
+#include <wx/aui/auibar.h>
+#endif
 #include "wx/event.h"
 #include "wx/notebook.h"
 #include "plugindata.h"
@@ -285,7 +299,7 @@ public:
 	 * \param parent toolbar parent, usually this is the main frame
 	 * \return toolbar or NULL
 	 */
-	virtual wxToolBar *CreateToolBar(wxWindow *parent) = 0;
+	virtual clToolBar *CreateToolBar(wxWindow *parent) = 0;
 
 	/**
 	 * Every plugin can place a sub menu in the 'Plugins' Menu at the menu bar
@@ -366,6 +380,11 @@ public:
 	}
 };
 
+#define CHECK_CL_SHUTDOWN(){\
+	if(m_mgr->IsShutdownInProgress())\
+		return;\
+}
+
 //Every dll must contain at least this function
 typedef IPlugin*    (*GET_PLUGIN_CREATE_FUNC)            (IManager*);
 typedef PluginInfo  (*GET_PLUGIN_INFO_FUNC)              ();
