@@ -24,7 +24,10 @@
 //////////////////////////////////////////////////////////////////////////////
 
 #include <wx/socket.h>
+#include "cl_registry.h"
+#include "fileextmanager.h"
 #include <wx/splash.h>
+#include <wx/fileconf.h>
 #include "evnvarlist.h"
 #include "environmentconfig.h"
 #include "conffilelocator.h"
@@ -50,6 +53,8 @@
 #include "wx/tokenzr.h"
 #include "wx/dir.h"
 #include <wx/stdpaths.h>
+#include "frame.h"
+#include "buidltab.h"
 
 #define __PERFORMANCE
 #include "performance.h"
@@ -58,7 +63,7 @@
 // Define the version string for this codelite
 //////////////////////////////////////////////
 extern wxChar *SvnRevision;
-wxString CODELITE_VERSION_STR = wxString::Format(wxT("v2.2.0.%s"), SvnRevision);
+wxString CODELITE_VERSION_STR = wxString::Format(wxT("v2.5.2.%s"), SvnRevision);
 
 #if defined(__WXMAC__)||defined(__WXGTK__)
 #include <signal.h> // sigprocmask
@@ -99,9 +104,9 @@ wxString MacGetBasePath()
 static wxBitmap clDrawSplashBitmap(const wxBitmap& bitmap, const wxString &mainTitle)
 {
 	wxBitmap bmp ( bitmap.GetWidth(), bitmap.GetHeight()  );
-    wxMemoryDC dcMem;
-	
-    dcMem.SelectObject( bmp );
+	wxMemoryDC dcMem;
+
+	dcMem.SelectObject( bmp );
 	dcMem.DrawBitmap  ( bitmap, 0, 0, true);
 
 	//write the main title & subtitle
@@ -116,7 +121,7 @@ static wxBitmap clDrawSplashBitmap(const wxBitmap& bitmap, const wxString &mainT
 
 	//draw shadow
 	dcMem.SetTextForeground(wxT("WHITE"));
-	
+
 	wxCoord textX = (bmpW - w)/2;
 	dcMem.DrawText(mainTitle, textX, 11);
 	dcMem.SelectObject(wxNullBitmap);
@@ -129,8 +134,7 @@ static wxBitmap clDrawSplashBitmap(const wxBitmap& bitmap, const wxString &mainT
 class clSplashScreen : public wxSplashScreen
 {
 public:
-	clSplashScreen(const wxBitmap& bmp) : wxSplashScreen(bmp, wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT, 5000, NULL, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxBORDER_SIMPLE| wxFRAME_NO_TASKBAR| wxSTAY_ON_TOP)
-	{
+	clSplashScreen(const wxBitmap& bmp) : wxSplashScreen(bmp, wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT, 5000, NULL, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxBORDER_SIMPLE| wxFRAME_NO_TASKBAR| wxSTAY_ON_TOP) {
 	}
 };
 
@@ -138,6 +142,7 @@ public:
 static const wxCmdLineEntryDesc cmdLineDesc[] = {
 	{wxCMD_LINE_SWITCH, wxT("v"), wxT("version"), wxT("Print current version"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
 	{wxCMD_LINE_SWITCH, wxT("h"), wxT("help"), wxT("Print usage"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
+	{wxCMD_LINE_SWITCH, wxT("p"), wxT("no-plugins"), wxT("Start codelite without any plugins"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
 	{wxCMD_LINE_OPTION, wxT("l"), wxT("line"), wxT("Open the file at a given line number"), wxCMD_LINE_VAL_NUMBER, wxCMD_LINE_PARAM_OPTIONAL },
 	{wxCMD_LINE_OPTION, wxT("b"), wxT("basedir"),  wxT("Use this path as CodeLite installation path"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
 	{wxCMD_LINE_PARAM,  NULL, NULL, wxT("Input file"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_MULTIPLE|wxCMD_LINE_PARAM_OPTIONAL },
@@ -147,6 +152,7 @@ static const wxCmdLineEntryDesc cmdLineDesc[] = {
 static const wxCmdLineEntryDesc cmdLineDesc[] = {
 	{wxCMD_LINE_SWITCH, "v", "version", "Print current version", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
 	{wxCMD_LINE_SWITCH, "h", "help", "Print usage", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
+	{wxCMD_LINE_SWITCH, "p", "no-plugins", "Start codelite without any plugins", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
 	{wxCMD_LINE_OPTION, "l", "line", "Open the file at a given line number", wxCMD_LINE_VAL_NUMBER, wxCMD_LINE_PARAM_OPTIONAL },
 	{wxCMD_LINE_OPTION, "b", "basedir",  "The base directory of CodeLite", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
 	{wxCMD_LINE_PARAM,  NULL, NULL, "Input file", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_MULTIPLE|wxCMD_LINE_PARAM_OPTIONAL },
@@ -164,15 +170,48 @@ static void massCopy(const wxString &sourceDir, const wxString &spec, const wxSt
 	}
 }
 
+#ifdef __WXGTK__
+//-------------------------------------------
+// Signal Handlers for GTK
+//-------------------------------------------
+static void WaitForDebugger(int signo)
+{
+	wxString msg;
+	msg << wxT("codelite crashed: you may attach to it using gdb\n")
+		<< wxT("or let it crash silently..\n")
+		<< wxT("Attach debugger?\n");
+
+	int rc = wxMessageBox(msg, wxT("CodeLite Crash Handler"), wxYES_NO|wxCENTER|wxICON_ERROR);
+	if(rc == wxYES) {
+
+		// Launch a shell command with the following command:
+		// gdb -p <PID>
+
+		char command[256];
+		memset (command, 0, sizeof(command));
+		sprintf(command, "xterm -T 'gdb' -e 'gdb -p %d'", getpid());
+		if(system (command) == 0) {
+			signal(signo, SIG_DFL);
+			raise(signo);
+		} else {
+			// Go down without launching the debugger, ask the user to do it manually
+			wxMessageBox(wxString::Format(wxT("Failed to launch the debugger\nYou may still attach to codelite manually by typing this command in a terminal:\ngdb -p %d"), getpid()), wxT("CodeLite Crash Handler"), wxOK|wxCENTER|wxICON_ERROR);
+			pause();
+		}
+	}
+
+	signal(signo, SIG_DFL);
+	raise(signo);
+}
+#endif
+
 IMPLEMENT_APP(App)
-BEGIN_EVENT_TABLE(App, wxApp)
-	//EVT_COMMAND(wxID_ANY, HIDE_SPLASH_EVENT_ID, App::OnHideSplash)
-END_EVENT_TABLE()
 
 extern void InitXmlResource();
 App::App(void)
 		: m_pMainFrame(NULL)
 		, m_singleInstance(NULL)
+		, m_loadPlugins(true)
 #ifdef __WXMSW__
 		, m_handler(NULL)
 #endif
@@ -202,6 +241,12 @@ bool App::OnInit()
 	sigemptyset( &mask_set );
 	sigaddset(&mask_set, SIGPIPE);
 	sigprocmask(SIG_SETMASK, &mask_set, NULL);
+#ifdef __WXGTK__
+	// Insall signal handlers
+	signal(SIGSEGV, WaitForDebugger);
+	signal(SIGABRT, WaitForDebugger);
+#endif
+
 #endif
 
 	wxSocketBase::Initialize();
@@ -237,12 +282,17 @@ bool App::OnInit()
 		return false;
 	}
 
-	if(parser.Found(wxT("h"))){
+	if (parser.Found(wxT("h"))) {
 		// print usage
 		parser.Usage();
 		return false;
 	}
 
+	if (parser.Found(wxT("p"))) {
+		// Load codelite without plugins
+		SetLoadPlugins(false);
+	}
+
 	wxString newBaseDir(wxEmptyString);
 	if (parser.Found(wxT("b"), &newBaseDir)) {
 		homeDir = newBaseDir;
@@ -254,14 +304,15 @@ bool App::OnInit()
 		homeDir = wxStandardPaths::Get().GetUserDataDir(); // ~/Library/Application Support/codelite or ~/.codelite
 
 		//Create the directory structure
-		wxMkDir(homeDir.ToAscii(), 0777);
-		wxMkDir((homeDir + wxT("/lexers/")).ToAscii(), 0777);
-		wxMkDir((homeDir + wxT("/lexers/Default")).ToAscii(), 0777);
-		wxMkDir((homeDir + wxT("/lexers/BlackTheme")).ToAscii(), 0777);
-		wxMkDir((homeDir + wxT("/rc/")).ToAscii(), 0777);
-		wxMkDir((homeDir + wxT("/images/")).ToAscii(), 0777);
-		wxMkDir((homeDir + wxT("/templates/")).ToAscii(), 0777);
-		wxMkDir((homeDir + wxT("/config/")).ToAscii(), 0777);
+		wxLogNull noLog;
+		wxMkdir(homeDir);
+		wxMkdir(homeDir + wxT("/lexers/"));
+		wxMkdir(homeDir + wxT("/lexers/Default"));
+		wxMkdir(homeDir + wxT("/lexers/BlackTheme"));
+		wxMkdir(homeDir + wxT("/rc/"));
+		wxMkdir(homeDir + wxT("/images/"));
+		wxMkdir(homeDir + wxT("/templates/"));
+		wxMkdir(homeDir + wxT("/config/"));
 
 		//copy the settings from the global location if needed
 		wxString installPath( INSTALL_DIR, wxConvUTF8 );
@@ -276,16 +327,21 @@ bool App::OnInit()
 
 #elif defined (__WXMAC__)
 	SetAppName(wxT("codelite"));
-	homeDir = wxGetHomeDir() + wxT("/.codelite/");
-	//Create the directory structure
-	wxMkDir(homeDir.ToAscii(), 0777);
-	wxMkDir((homeDir + wxT("/lexers/")).ToAscii(), 0777);
-	wxMkDir((homeDir + wxT("/lexers/Default")).ToAscii(), 0777);
-	wxMkDir((homeDir + wxT("/lexers/BlackTheme")).ToAscii(), 0777);
-	wxMkDir((homeDir + wxT("/rc/")).ToAscii(), 0777);
-	wxMkDir((homeDir + wxT("/images/")).ToAscii(), 0777);
-	wxMkDir((homeDir + wxT("/templates/")).ToAscii(), 0777);
-	wxMkDir((homeDir + wxT("/config/")).ToAscii(), 0777);
+	homeDir = wxStandardPaths::Get().GetUserDataDir();
+
+	{
+		wxLogNull noLog;
+
+		//Create the directory structure
+		wxMkdir(homeDir);
+		wxMkdir(homeDir + wxT("/lexers/"));
+		wxMkdir(homeDir + wxT("/lexers/Default"));
+		wxMkdir(homeDir + wxT("/lexers/BlackTheme"));
+		wxMkdir(homeDir + wxT("/rc/"));
+		wxMkdir(homeDir + wxT("/images/"));
+		wxMkdir(homeDir + wxT("/templates/"));
+		wxMkdir(homeDir + wxT("/config/"));
+	}
 
 	wxString installPath( MacGetBasePath() );
 	ManagerST::Get()->SetInstallDir( installPath );
@@ -300,13 +356,13 @@ bool App::OnInit()
 
 	// try to locate the menu/rc.xrc file
 	wxFileName fn(homeDir + wxT("/rc"), wxT("menu.xrc"));
-	if(!fn.FileExists()){
+	if (!fn.FileExists()) {
 		// we got wrong home directory
 		wxFileName appFn( wxAppBase::argv[0] );
 		homeDir = appFn.GetPath();
 	}
 
-	if(fnHomdDir.IsRelative()){
+	if (fnHomdDir.IsRelative()) {
 		fnHomdDir.MakeAbsolute();
 		homeDir = fnHomdDir.GetPath();
 	}
@@ -315,7 +371,7 @@ bool App::OnInit()
 #endif
 
 	// Update codelite revision and Version
-	EditorConfig::Init( SvnRevision, wxT("2.0.2") );
+	EditorConfigST::Get()->Init(SvnRevision, wxT("2.0.2") );
 
 	wxString curdir = wxGetCwd();
 	::wxSetWorkingDirectory(homeDir);
@@ -359,63 +415,34 @@ bool App::OnInit()
 		return false;
 	}
 
+	//---------------------------------------------------------
+	// Set environment variable for CodeLiteDir (make it first
+	// on the list so it can be used by other variables)
+	//---------------------------------------------------------
+	EvnVarList vars;
+	EnvironmentConfig::Instance()->Load();
+	EnvironmentConfig::Instance()->ReadObject(wxT("Variables"), &vars);
+
+	vars.InsertVariable(wxT("Default"), wxT("CodeLiteDir"), ManagerST::Get()->GetInstallDir());
+	EnvironmentConfig::Instance()->WriteObject(wxT("Variables"), &vars);
+
+	//---------------------------------------------------------
+
 #ifdef __WXMSW__
-	// Update PATH environment variable with the install directory and
-	// MinGW default installation (if exists)
-	wxString pathEnv;
-	if(wxGetEnv(wxT("PATH"), &pathEnv) == false){
-		wxLogMessage(_("WARNING: Failed to load environment variable PATH!"));
-	} else {
-		pathEnv << wxT(";") << homeDir << wxT(";");
 
-		// read the installation path of MinGW & WX
-		wxRegKey rk(wxT("HKEY_CURRENT_USER\\Software\\CodeLite"));
-		if(rk.Exists()) {
-			m_parserPaths.Clear();
-			wxString strWx, strMingw;
-			if(rk.HasValue(wxT("wx"))){
-				rk.QueryValue(wxT("wx"), strWx);
-			}
+	// Read registry values
+	MSWReadRegistry();
 
-			if(rk.HasValue(wxT("mingw"))){
-				rk.QueryValue(wxT("mingw"), strMingw);
-			}
-
-			EvnVarList vars;
-			EnvironmentConfig::Instance()->Load();
-			EnvironmentConfig::Instance()->ReadObject(wxT("Variables"), &vars);
-
-			if(strWx.IsEmpty() == false) {
-				// we have WX installed on this machine, set the path of WXWIN & WXCFG to point to it
-				std::map<wxString, wxString> envs = vars.GetVariables(wxT("Default"));
-				if(envs.find(wxT("WXWIN")) == envs.end())
-					vars.AddVariable(wxT("Default"), wxT("WXWIN"), strWx);
-
-				if(envs.find(wxT("WXCFG")) == envs.end())
-					vars.AddVariable(wxT("Default"), wxT("WXCFG"), wxT("gcc_dll\\mswu"));
-
-				EnvironmentConfig::Instance()->WriteObject(wxT("Variables"), &vars);
-				wxSetEnv(wxT("WX_INCL_HOME"), strWx + wxT("\\include"));
-			}
-
-			if(strMingw.IsEmpty() == false) {
-				// Add the installation include paths
-				pathEnv << wxT(";") << strMingw << wxT("\\bin");
-				wxSetEnv(wxT("MINGW_INCL_HOME"), strMingw);
-			}
-		}
-
-		if(wxSetEnv(wxT("PATH"), pathEnv) == false){
-			wxLogMessage(_("WARNING: Failed to update environment variable PATH"));
-		}
-	}
 #endif
 
 	GeneralInfo inf;
 	cfg->ReadObject(wxT("GeneralInfo"), &inf);
 
-	bool showSplash = inf.GetFlags() & CL_SHOW_SPLASH ? true : false;
-	
+	bool showSplash(false);
+#ifndef __WXDEBUG__
+	showSplash = inf.GetFlags() & CL_SHOW_SPLASH ? true : false;
+#endif
+
 	m_splash = NULL;
 	if (showSplash) {
 		wxBitmap bitmap;
@@ -426,7 +453,7 @@ bool App::OnInit()
 			m_splash = new clSplashScreen(splash);
 		}
 	}
-	
+
 	// Create the main application window (a dialog in this case)
 	// NOTE: Vertical dimension comprises the caption bar.
 	//       Horizontal dimension has to take into account the thin
@@ -442,9 +469,9 @@ bool App::OnInit()
 
 	long lineNumber(0);
 	parser.Found(wxT("l"), &lineNumber);
-	if(lineNumber > 0){
+	if (lineNumber > 0) {
 		lineNumber--;
-	}else{
+	} else {
 		lineNumber = 0;
 	}
 
@@ -464,43 +491,42 @@ bool App::OnInit()
 
 	wxLogMessage(wxString::Format(wxT("Install path: %s"), ManagerST::Get()->GetInstallDir().c_str()));
 	wxLogMessage(wxString::Format(wxT("Startup Path: %s"), ManagerST::Get()->GetStarupDirectory().c_str()));
+
+#ifdef __WXGTK__
+	// Needed on GTK
+	ManagerST::Get()->UpdateMenuAccelerators();
+	if (Frame::Get()->GetMainBook()->GetActiveEditor() == NULL) {
+		Frame::Get()->GetOutputPane()->GetBuildTab()->SetFocus();
+	}
+#endif
+
 	return TRUE;
 }
 
 int App::OnExit()
 {
+	EditorConfigST::Free();
 	ConfFileLocator::Release();
 	return 0;
 }
 
 bool App::CopySettings(const wxString &destDir, wxString& installPath)
 {
-	bool fileExist = wxFileName::FileExists( destDir + wxT("/config/codelite.xml") );
-	bool copyAnyways(true);
+	wxLogNull noLog;
 
-	if (fileExist) {
-		if (CheckRevision(destDir + wxT("/config/codelite.xml")) == true) {
-			//revision is ok
-			copyAnyways = false;
-		}
-	}
-
-	if ( !fileExist || copyAnyways ) {
-		///////////////////////////////////////////////////////////////////////////////////////////
-		// copy new settings from the global installation location which is currently located at
-		// /usr/local/share/codelite/ (Linux) or at codelite.app/Contents/SharedSupport
-		///////////////////////////////////////////////////////////////////////////////////////////
-		CopyDir(installPath + wxT("/templates/"), destDir + wxT("/templates/"));
-		CopyDir(installPath + wxT("/lexers/"), destDir + wxT("/lexers/"));
-		massCopy  (installPath + wxT("/images/"), wxT("*.png"), destDir + wxT("/images/"));
-		massCopy  (installPath + wxT("/"), wxT("*.tags"), destDir + wxT("/"));
-		wxCopyFile(installPath + wxT("/config/codelite.xml.default"), destDir + wxT("/config/codelite.xml.default"));
-		wxCopyFile(installPath + wxT("/rc/menu.xrc"), destDir + wxT("/rc/menu.xrc"));
-		wxCopyFile(installPath + wxT("/index.html"), destDir + wxT("/index.html"));
-		wxCopyFile(installPath + wxT("/svnreport.html"), destDir + wxT("/svnreport.html"));
-		wxCopyFile(installPath + wxT("/astyle.sample"), destDir + wxT("/astyle.sample"));
-		wxCopyFile(installPath + wxT("/config/accelerators.conf.default"), destDir + wxT("/config/accelerators.conf.default"));
-	}
+	///////////////////////////////////////////////////////////////////////////////////////////
+	// copy new settings from the global installation location which is currently located at
+	// /usr/local/share/codelite/ (Linux) or at codelite.app/Contents/SharedSupport
+	///////////////////////////////////////////////////////////////////////////////////////////
+	CopyDir(installPath + wxT("/templates/"), destDir + wxT("/templates/"));
+	CopyDir(installPath + wxT("/lexers/"), destDir + wxT("/lexers/"));
+	massCopy  (installPath + wxT("/images/"), wxT("*.png"), destDir + wxT("/images/"));
+	wxCopyFile(installPath + wxT("/config/codelite.xml.default"), destDir + wxT("/config/codelite.xml.default"));
+	wxCopyFile(installPath + wxT("/rc/menu.xrc"), destDir + wxT("/rc/menu.xrc"));
+	wxCopyFile(installPath + wxT("/index.html"), destDir + wxT("/index.html"));
+	wxCopyFile(installPath + wxT("/svnreport.html"), destDir + wxT("/svnreport.html"));
+	wxCopyFile(installPath + wxT("/astyle.sample"), destDir + wxT("/astyle.sample"));
+	wxCopyFile(installPath + wxT("/config/accelerators.conf.default"), destDir + wxT("/config/accelerators.conf.default"));
 	return true;
 }
 
@@ -520,37 +546,6 @@ void App::OnFatalException()
 #endif
 }
 
-void App::OnIdle(wxIdleEvent &e)
-{
-	//delegate the event to the manager class
-	Frame::Get()->ProcessEvent(e);
-	e.Skip();
-}
-
-void App::OnHideSplash(wxCommandEvent &e)
-{
-	m_splash->Close(true);
-	m_splash->Destroy();
-	e.Skip();
-}
-
-bool App::CheckRevision(const wxString &fileName)
-{
-	wxXmlDocument doc;
-	doc.Load(fileName);
-	if (doc.IsOk()) {
-		wxXmlNode *root = doc.GetRoot();
-		if (root) {
-			wxString configRevision = XmlUtils::ReadString(root, wxT("Revision"));
-			wxString curRevision(SvnRevision);
-			if (configRevision.Trim().Trim(false) == curRevision.Trim().Trim(false)) {
-				return true;
-			}
-		}
-	}
-	return false;
-}
-
 bool App::CheckSingularity(const wxCmdLineParser &parser, const wxString &curdir)
 {
 	// check for single instance
@@ -577,10 +572,10 @@ bool App::CheckSingularity(const wxCmdLineParser &parser, const wxString &curdir
 
 				wxString file_name, tmp_file;
 				tmp_file 	<< ManagerST::Get()->GetStarupDirectory()
-							<< wxT("/ipc/command.msg.tmp");
+				<< wxT("/ipc/command.msg.tmp");
 
 				file_name 	<< ManagerST::Get()->GetStarupDirectory()
-							<< wxT("/ipc/command.msg");
+				<< wxT("/ipc/command.msg");
 
 				// write the content to a temporary file, once completed,
 				// rename the file to the actual file name
@@ -593,3 +588,159 @@ bool App::CheckSingularity(const wxCmdLineParser &parser, const wxString &curdir
 	return true;
 }
 
+void App::MacOpenFile(const wxString& fileName)
+{
+	switch (FileExtManager::GetType(fileName)) {
+	case FileExtManager::TypeWorkspace:
+		ManagerST::Get()->OpenWorkspace(fileName);
+		break;
+	default:
+		Frame::Get()->GetMainBook()->OpenFile(fileName);
+		break;
+	}
+}
+
+void App::MSWReadRegistry()
+{
+#ifdef __WXMSW__
+
+	EvnVarList vars;
+	EnvironmentConfig::Instance()->Load();
+	EnvironmentConfig::Instance()->ReadObject(wxT("Variables"), &vars);
+
+	/////////////////////////////////////////////////////////////////
+	// New way of registry:
+	// Read the registry INI file
+	/////////////////////////////////////////////////////////////////
+
+	// Update PATH environment variable with the install directory and
+	// MinGW default installation (if exists)
+	wxString pathEnv;
+	wxGetEnv(wxT("PATH"), &pathEnv);
+	pathEnv << wxT(";") << ManagerST::Get()->GetInstallDir() << wxT(";");
+
+	// Load the registry file
+	wxString iniFile;
+	iniFile << ManagerST::Get()->GetInstallDir() << wxFileName::GetPathSeparator() << wxT("registry.ini");
+
+	if (wxFileName::FileExists(iniFile)) {
+		clRegistry::SetFilename(iniFile);
+		clRegistry registry;
+
+		m_parserPaths.Clear();
+		wxString strWx, strMingw, strUnitTestPP;
+		registry.Read(wxT("wx"),         strWx);
+		registry.Read(wxT("mingw"),      strMingw);
+		registry.Read(wxT("unittestpp"), strUnitTestPP);
+
+		// Supprot for wxWidgets
+		if (strWx.IsEmpty() == false) {
+			// we have WX installed on this machine, set the path of WXWIN & WXCFG to point to it
+			EnvMap envs = vars.GetVariables(wxT("Default"));
+
+			if (!envs.Contains(wxT("WXWIN"))) {
+				vars.AddVariable(wxT("Default"), wxT("WXWIN"), strWx);
+				vars.AddVariable(wxT("Default"), wxT("PATH"),  wxT("$(WXWIN)\\lib\\gcc_dll;$(PATH)"));
+			}
+
+			if (!envs.Contains(wxT("WXCFG")))
+				vars.AddVariable(wxT("Default"), wxT("WXCFG"), wxT("gcc_dll\\mswu"));
+
+			EnvironmentConfig::Instance()->WriteObject(wxT("Variables"), &vars);
+			wxSetEnv(wxT("WX_INCL_HOME"), strWx + wxT("\\include"));
+		}
+
+		// Support for UnitTest++
+		if (strUnitTestPP.IsEmpty() == false) {
+			// we have UnitTest++ installed on this machine
+			EnvMap envs = vars.GetVariables(wxT("Default"));
+
+			if (!envs.Contains(wxT("UNIT_TEST_PP_SRC_DIR")) ) {
+				vars.AddVariable(wxT("Default"), wxT("UNIT_TEST_PP_SRC_DIR"), strUnitTestPP);
+			}
+
+			EnvironmentConfig::Instance()->WriteObject(wxT("Variables"), &vars);
+		}
+
+		// Support for MinGW
+		if (strMingw.IsEmpty() == false) {
+			// Add the installation include paths
+			pathEnv << wxT(";") << strMingw << wxT("\\bin");
+			wxSetEnv(wxT("MINGW_INCL_HOME"), strMingw);
+		}
+
+		wxSetEnv(wxT("PATH"), pathEnv);
+
+	} else {
+
+		// Use the old way: read values from registry. This code is kept so people using
+		// SVN build of codelite can continue working with it until an official build is released
+		// Update PATH environment variable with the install directory and
+		// MinGW default installation (if exists)
+		wxString pathEnv;
+		if (wxGetEnv(wxT("PATH"), &pathEnv) == false) {
+			wxLogMessage(_("WARNING: Failed to load environment variable PATH!"));
+		} else {
+			pathEnv << wxT(";") << ManagerST::Get()->GetInstallDir() << wxT(";");
+
+			// read the installation path of MinGW & WX
+			wxRegKey rk(wxT("HKEY_CURRENT_USER\\Software\\CodeLite"));
+			if (rk.Exists()) {
+				m_parserPaths.Clear();
+				wxString strWx, strMingw, strUnitTestPP;
+				if (rk.HasValue(wxT("wx"))) {
+					rk.QueryValue(wxT("wx"), strWx);
+				}
+
+				if (rk.HasValue(wxT("mingw"))) {
+					rk.QueryValue(wxT("mingw"), strMingw);
+				}
+
+				if (rk.HasValue(wxT("unittestpp"))) {
+					rk.QueryValue(wxT("unittestpp"), strUnitTestPP);
+				}
+
+
+
+				if (strWx.IsEmpty() == false) {
+					// we have WX installed on this machine, set the path of WXWIN & WXCFG to point to it
+					EnvMap envs = vars.GetVariables(wxT("Default"));
+
+					if (envs.Contains(wxT("WXWIN")) == false) {
+						vars.AddVariable(wxT("Default"), wxT("WXWIN"), strWx);
+						vars.AddVariable(wxT("Default"), wxT("PATH"),  wxT("$(WXWIN)\\lib\\gcc_dll;$(PATH)"));
+					}
+
+					if (envs.Contains(wxT("WXCFG")) == false)
+						vars.AddVariable(wxT("Default"), wxT("WXCFG"), wxT("gcc_dll\\mswu"));
+
+					EnvironmentConfig::Instance()->WriteObject(wxT("Variables"), &vars);
+					wxSetEnv(wxT("WX_INCL_HOME"), strWx + wxT("\\include"));
+				}
+
+				if (strMingw.IsEmpty() == false) {
+					// Add the installation include paths
+					pathEnv << wxT(";") << strMingw << wxT("\\bin");
+					wxSetEnv(wxT("MINGW_INCL_HOME"), strMingw);
+				}
+
+				// Support for UnitTest++
+				if (strUnitTestPP.IsEmpty() == false) {
+					// we have UnitTest++ installed on this machine
+					EnvMap envs = vars.GetVariables(wxT("Default"));
+
+					if (!envs.Contains(wxT("UNIT_TEST_PP_SRC_DIR")) ) {
+						vars.AddVariable(wxT("Default"), wxT("UNIT_TEST_PP_SRC_DIR"), strUnitTestPP);
+					}
+
+					EnvironmentConfig::Instance()->WriteObject(wxT("Variables"), &vars);
+				}
+			}
+
+			if (wxSetEnv(wxT("PATH"), pathEnv) == false) {
+				wxLogMessage(_("WARNING: Failed to update environment variable PATH"));
+			}
+		}
+	}
+#endif
+}
