@@ -23,6 +23,7 @@
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 #include "precompiled_header.h"
+#include <wx/toolbook.h>
 #include <wx/ffile.h>
 #include "globals.h"
 #include "plugin.h"
@@ -184,6 +185,7 @@ void BuildTab::Clear()
 	m_errorCount = 0;
 	m_warnCount = 0;
 	m_cmp.Reset ( NULL );
+	m_baseDir.Clear();
 	Frame::Get()->GetOutputPane()->GetErrorsTab()->ClearLines();
 	LEditor *editor = Frame::Get()->GetMainBook()->GetActiveEditor();
 	if ( editor ) {
@@ -233,16 +235,23 @@ bool BuildTab::ExtractLineInfo ( LineInfo &info, const wxString &text, const wxR
         // find the actual workspace file (if possible)
 		wxString filename = text.Mid( info.filestart, info.filelen).Trim().Trim(false);
 		wxFileName fn(filename);
-		
+
+		info.rawFilename = filename;
+
 		// Use the current base dir
-		wxString baseDir ( m_baseDir );
+		wxString baseDir;
+		if( m_baseDir.IsEmpty() == false ) {
+			// Use the last collected directory
+			baseDir = m_baseDir.Last();
+		}
+
 		if(baseDir.IsEmpty()) {
 			ProjectPtr project = ManagerST::Get()->GetProject(info.project);
 			if(project) {
 				baseDir = project->GetFileName().GetPath();
 			}
 		}
-		
+
 		fn.Normalize(wxPATH_NORM_ALL & ~wxPATH_NORM_LONG, baseDir);
 		info.filename = fn.GetFullPath();
 	}
@@ -290,7 +299,28 @@ void BuildTab::DoMarkAndOpenFile ( std::map<int,LineInfo>::iterator i, bool scro
 		filename.Normalize(wxPATH_NORM_ALL & ~wxPATH_NORM_LONG);
 	}
 
-	LEditor *editor = Frame::Get()->GetMainBook()->OpenFile ( filename.GetFullPath(), info.project, info.linenum );
+	// Make sure that the file actually exists
+	wxString file (filename.GetFullPath());
+	if(!filename.FileExists() && m_baseDir.IsEmpty() == false) {
+
+		// When using custom build + multicore, the messages of 'Entering directory' are printed randomly
+		// and may break the open file logic
+		// What we do here, is we loop over the m_baseDir and trying to locate a file that does actually exist
+		// using all the collected 'Entering directory' messages found so far.
+		// we start looping from the end of the array
+		for(size_t i=m_baseDir.GetCount()-1; i >= 0; i--) {
+			wxString tmpFilename = info.rawFilename;
+			wxFileName fn(tmpFilename);
+
+			fn.Normalize(wxPATH_NORM_ALL & ~wxPATH_NORM_LONG, m_baseDir.Item(i));
+			if(fn.FileExists()) {
+				file = fn.GetFullPath();
+				break;
+			}
+		}
+	}
+
+	LEditor *editor = Frame::Get()->GetMainBook()->OpenFile ( file, info.project, info.linenum );
     if (editor == NULL) {
 		return;
 	}
@@ -329,10 +359,10 @@ void BuildTab::MarkEditor ( LEditor *editor )
 	std::multimap<wxString,int>::iterator e = iters.second;
     if (b == m_fileMap.end())
         return;
-		
+
 	std::set<wxString> uniqueSet;
     for (; b != e; b++ ) {
-		
+
 		// get the line info related to the BuildTab's line number (b->second)
         std::map<int,LineInfo>::iterator i = m_lineInfo.find ( b->second ) ;
 
@@ -344,8 +374,8 @@ void BuildTab::MarkEditor ( LEditor *editor )
 
 			wxMemoryBuffer style_bytes;
 			LineInfo lineInfo = i->second;
-			
-			// For performance, dont add the exact same markers to the same line number/filename 
+
+			// For performance, dont add the exact same markers to the same line number/filename
 			// with the exact same tip
 			wxString tipMagic;
 			tipMagic << lineInfo.linenum << lineInfo.linetext;
@@ -357,13 +387,13 @@ void BuildTab::MarkEditor ( LEditor *editor )
 				// add it to the unique set
 				uniqueSet.insert(tipMagic);
 			}
-			
+
 			// format the tip
 			int line_number = lineInfo.linenum;
 			wxString tip = GetBuildToolTip(editor->GetFileName().GetFullPath(), line_number, style_bytes);
-						
+
 			// Set annotations
-			if ( options.GetErrorWarningStyle() & BuildTabSettingsData::EWS_Annotations ) {
+			if ( line_number >= 0 && options.GetErrorWarningStyle() & BuildTabSettingsData::EWS_Annotations ) {
 				editor->AnnotationSetText (line_number, tip);
 				editor->AnnotationSetStyles(line_number, style_bytes );
 			}
@@ -406,12 +436,12 @@ void BuildTab::OnRepeatOutputUI ( wxUpdateUIEvent& e )
 void BuildTab::OnBuildStarted ( wxCommandEvent &e )
 {
 	e.Skip();
-	
+
 	m_building = true;
-	
+
 	// Clear all compiler parsing information
 	m_compilerParseInfo.clear();
-		
+
 	// Loop over all known compilers and cache the regular expressions
 	BuildSettingsConfigCookie cookie;
 	CompilerPtr cmp = BuildSettingsConfigST::Get()->GetFirstCompiler(cookie);
@@ -421,39 +451,44 @@ void BuildTab::OnBuildStarted ( wxCommandEvent &e )
 		const Compiler::CmpListInfoPattern& warnPatterns = cmp->GetWarnPatterns();
 		Compiler::CmpListInfoPattern::const_iterator iter;
 		for (iter = errPatterns.begin(); iter != errPatterns.end(); iter++) {
-			
+
 			CompiledPatternPtr compiledPatternPtr(new CompiledPattern(new wxRegEx(iter->pattern), iter->fileNameIndex, iter->lineNumberIndex));
 			if(compiledPatternPtr->regex->IsValid()) {
 				cmpPatterns.errorsPatterns.push_back( compiledPatternPtr );
 			}
 		}
-		
+
 		for (iter = warnPatterns.begin(); iter != warnPatterns.end(); iter++) {
-			
+
 			CompiledPatternPtr compiledPatternPtr(new CompiledPattern(new wxRegEx(iter->pattern), iter->fileNameIndex, iter->lineNumberIndex));
 			if(compiledPatternPtr->regex->IsValid()) {
 				cmpPatterns.warningPatterns.push_back( compiledPatternPtr );
 			}
 		}
-		
+
 		m_compilerParseInfo[cmp->GetName()] = cmpPatterns;
 		cmp =  BuildSettingsConfigST::Get()->GetNextCompiler(cookie);
 	}
-	
+
 	if ( e.GetEventType() != wxEVT_SHELL_COMMAND_STARTED_NOCLEAN ) {
 		// Reset output and counters
 		Clear();
 	}
-	
+
 	AppendText ( BUILD_START_MSG );
-//	Frame::Get()->SetStatusMessage ( e.GetString(), 3, XRCID ( "build" ) );
     OutputPane *opane = Frame::Get()->GetOutputPane();
+
+	wxWindow *win(NULL);
+	int sel =  opane->GetNotebook()->GetSelection();
+	if(sel != wxNOT_FOUND)
+		win = opane->GetNotebook()->GetPage((size_t)sel);
+
 	if (m_showMe == BuildTabSettingsData::ShowOnEnd &&
             m_autoHide &&
             ManagerST::Get()->IsPaneVisible(opane->GetCaption()) &&
-            (opane->GetNotebook()->GetCurrentPage() == this ||
-                opane->GetNotebook()->GetCurrentPage() == opane->GetErrorsTab())) {
-        // user prefers to see build/errors tabs only at end of unsuccessful build
+			(win == this || win == opane->GetErrorsTab()))
+	{
+		// user prefers to see build/errors tabs only at end of unsuccessful build
         ManagerST::Get()->HidePane(opane->GetName());
     }
 	m_sw.Start();
@@ -515,7 +550,7 @@ void BuildTab::OnBuildEnded ( wxCommandEvent &e )
 	}
 
 	MarkEditor ( Frame::Get()->GetMainBook()->GetActiveEditor() );
-	
+
 	// notify the plugins that the build had ended
 	PostCmdEvent(wxEVT_BUILD_ENDED);
 }
@@ -624,8 +659,16 @@ wxString BuildTab::GetBuildToolTip(const wxString& fileName, int lineno, wxMemor
 			wxString asciiTip;
 
 			for(size_t at=0; at<tmpTip.Length(); at++) {
-				if( isprint((char)tmpTip.GetChar(at) ) || tmpTip.GetChar(at) == wxT('\n') ) {
-					asciiTip.Append( tmpTip.GetChar(at) );
+	#if wxVERSION_NUMBER < 2900
+				wxChar c = tmpTip.GetChar(at);
+	#else
+				wxUniChar c = tmpTip.GetChar(at);
+				if ( !c.IsAscii() )	{
+					continue;
+				}
+	#endif
+				if( wxIsprint(c) || c == wxT('\n') ) {
+					asciiTip.Append(c);
 				}
 			}
 
@@ -700,43 +743,45 @@ void BuildTab::DoProcessLine(const wxString& text, int lineno)
 	if ( text.StartsWith ( BUILD_START_MSG ) || text.StartsWith ( BUILD_END_MSG ) ) {
 		info.linecolor = wxSCI_LEX_GCC_BUILDING;
 	}
-	
+
 	// Check for makefile directory changes lines
 	if(text.Contains(wxT("Entering directory `"))) {
 		wxString currentDir = text.AfterFirst(wxT('`'));
 		currentDir = currentDir.BeforeLast(wxT('\''));
-		m_baseDir = currentDir;
+
+		// Collect the m_baseDir
+		m_baseDir.Add(currentDir);
 	}
-	
+
 	if ( info.linecolor == wxSCI_LEX_GCC_BUILDING || !m_cmp ) {
 		// no more line info to get
 	} else {
-		
-		// Find error first
-		bool isError = false;
-		
+
+		// Find *warnings* first
+		bool isWarning = false;
+
 		CompilerPatterns cmpPatterns;
 		if(!GetCompilerPatterns(m_cmp->GetName(), cmpPatterns)) {
 			return;
 		}
-		
-		for(size_t i=0; i<cmpPatterns.errorsPatterns.size(); i++) {
-			CompiledPatternPtr cmpPatterPtr = cmpPatterns.errorsPatterns.at(i);
+
+		// If it is not an error, maybe it's a warning
+		for(size_t i=0; i<cmpPatterns.warningPatterns.size(); i++) {
+			CompiledPatternPtr cmpPatterPtr = cmpPatterns.warningPatterns.at(i);
 			if ( ExtractLineInfo(info, text, *(cmpPatterPtr->regex), cmpPatterPtr->fileIndex, cmpPatterPtr->lineIndex) ) {
-				info.linecolor = wxSCI_LEX_GCC_ERROR;
-				m_errorCount++;
-				isError = true;
+				info.linecolor = wxSCI_LEX_GCC_WARNING;
+				m_warnCount++;
+				isWarning = true;
 				break;
 			}
 		}
-		if (!isError) {
-			// If it is not an error, maybe it's a warning
-			for(size_t i=0; i<cmpPatterns.warningPatterns.size(); i++) {
-				CompiledPatternPtr cmpPatterPtr = cmpPatterns.warningPatterns.at(i);
+
+		if ( !isWarning ) {
+			for(size_t i=0; i<cmpPatterns.errorsPatterns.size(); i++) {
+				CompiledPatternPtr cmpPatterPtr = cmpPatterns.errorsPatterns.at(i);
 				if ( ExtractLineInfo(info, text, *(cmpPatterPtr->regex), cmpPatterPtr->fileIndex, cmpPatterPtr->lineIndex) ) {
-					info.linecolor = wxSCI_LEX_GCC_WARNING;
-					m_warnCount++;
-					isError = true;
+					info.linecolor = wxSCI_LEX_GCC_ERROR;
+					m_errorCount++;
 					break;
 				}
 			}
