@@ -49,6 +49,7 @@
 class wxFindReplaceDialog;
 class CCBox;
 class clEditorTipWindow;
+class DisplayVariableDlg;
 
 enum sci_annotation_styles {
 	eAnnotationStyleError = 128, eAnnotationStyleWarning
@@ -133,7 +134,10 @@ class LEditor : public wxScintilla, public IEditor
 	bool                                        m_reloadingFile;
 	bool                                        m_disableSmartIndent;
 	bool                                        m_disableSemicolonShift;
-	clEditorTipWindow*                        m_functionTip;
+	clEditorTipWindow*                          m_functionTip;
+	wxChar                                      m_lastCharEntered;
+	int                                         m_lastCharEnteredPos;
+//	DisplayVariableDlg*                         m_debuggerTip;
 
 public:
 	static FindReplaceData &GetFindReplaceData() {
@@ -150,6 +154,11 @@ public:
 	clEditorTipWindow* GetFunctionTip() {
 		return m_functionTip;
 	}
+
+//	DisplayVariableDlg *GetDebuggerTip() {
+//		return m_debuggerTip;
+//	}
+
 public:
 	/// Construct a LEditor object
 	LEditor(wxWindow* parent);
@@ -522,6 +531,8 @@ public:
 	virtual int GetCurrentLine();
 	virtual void ReplaceSelection(const wxString &text);
 	virtual wxString GetSelection();
+	virtual int GetSelectionStart();
+	virtual int GetSelectionEnd();
 	virtual void SelectText(int startPos, int len);
 
 	virtual void SetLexerName(const wxString &lexerName);
