@@ -25,6 +25,8 @@
 
 
 #include "pluginmanager.h"
+#include "drawingutils.h"
+#include "buildtabsettingsdata.h"
 #include "cl_editor_tip_window.h"
 #include "implement_parent_virtual_functions.h"
 #include "debuggerasciiviewer.h"
@@ -60,19 +62,15 @@
 #include "frame.h"
 #include "debuggermanager.h"
 #include "addincludefiledlg.h"
-#include "variable.h"
-#include "function.h"
 #include "workspacetab.h"
 #include "fileview.h"
 #include "refactorindexbuildjob.h"
 #include "new_quick_watch_dlg.h"
+#include "code_completion_api.h"
 
 //#define __PERFORMANCE
 #include "performance.h"
 
-extern void get_variables(const std::string &in, VariableList &li, const std::map<std::string, std::string> &ignoreTokens, bool isUsedWithinFunc);
-extern void get_functions(const std::string &in, FunctionList &li, const std::map<std::string, std::string> &ignoreTokens);
-
 static bool IsSource(const wxString &ext)
 {
 	wxString e(ext);
@@ -231,10 +229,16 @@ void ContextCpp::OnDwellStart(wxScintillaEvent &event)
 		return;
 	}
 
+	int foundPos(wxNOT_FOUND);
+	if (rCtrl.PreviousChar(word_start, foundPos) == wxT('~'))
+		word.Prepend(wxT("~"));
+
 	//get the expression we are hovering over
 	wxString expr = GetExpression(end, false);
+
 	// get the full text of the current page
 	wxString text = rCtrl.GetTextRange(0, pos);
+
 	// now we are ready to process the scope and build our tips
 	std::vector<wxString> tips;
 	int line = rCtrl.LineFromPosition(rCtrl.GetCurrentPosition())+1;
@@ -319,8 +323,8 @@ void ContextCpp::AutoIndent(const wxChar &nChar)
 {
 	LEditor &rCtrl = GetCtrl();
 
-	if(rCtrl.GetDisableSmartIndent()) {
-		ContextBase::AutoIndent(nChar);
+	if (rCtrl.GetDisableSmartIndent()) {
+		/*ContextBase::AutoIndent(nChar);*/
 		return;
 	}
 
@@ -340,13 +344,20 @@ void ContextCpp::AutoIndent(const wxChar &nChar)
 
 		int      prevpos (wxNOT_FOUND);
 		int      foundPos(wxNOT_FOUND);
-		wxString word;
 
+		wxString word;
 		wxChar ch = rCtrl.PreviousChar(curpos, prevpos );
 		word      = rCtrl.PreviousWord(curpos, foundPos);
 
+		bool isPreLinePreProcessLine(false);
+		if (line) {
+			wxString lineStr = rCtrl.GetLine( line - 1 );
+			lineStr.Trim().Trim(false);
+			isPreLinePreProcessLine = lineStr.StartsWith(wxT("#"));
+		}
+
 		// user hit ENTER after 'else'
-		if ( word == wxT("else") ) {
+		if ( word == wxT("else") && !isPreLinePreProcessLine ) {
 			int prevLine = rCtrl.LineFromPosition(prevpos);
 			rCtrl.SetLineIndentation(line, rCtrl.GetIndent() + rCtrl.GetLineIndentation(prevLine));
 			rCtrl.SetCaretAt(rCtrl.GetLineIndentPosition(line));
@@ -429,7 +440,7 @@ void ContextCpp::AutoIndent(const wxChar &nChar)
 
 		int matchPos = wxNOT_FOUND;
 		wxChar previousChar = rCtrl.PreviousChar(rCtrl.PositionBefore(curpos), matchPos);
-		if(previousChar != wxT('{') && lineString == wxT("{")) {
+		if (previousChar != wxT('{') && lineString == wxT("{")) {
 			// indent this line accroding to the previous line
 			int line = rCtrl.LineFromPosition(rCtrl.GetCurrentPos());
 			rCtrl.SetLineIndentation(line, rCtrl.GetLineIndentation(line-1));
@@ -849,9 +860,12 @@ void ContextCpp::CompleteWord()
 	//get the full text of the current page
 	wxString text = rCtrl.GetTextRange(0, rCtrl.GetCurrentPos());
 	int lineNum = rCtrl.LineFromPosition(rCtrl.GetCurrentPosition())+1;
+
+	DoSetProjectPaths();
 	if (mgr->WordCompletionCandidates(rCtrl.GetFileName(), lineNum, expr, text, word, candidates)) {
 		DisplayCompletionBox(candidates, word, false);
 	}
+
 }
 
 void ContextCpp::DisplayCompletionBox(const std::vector<TagEntryPtr> &tags, const wxString &word, bool showFullDecl)
@@ -1276,7 +1290,7 @@ void ContextCpp::OnGenerateSettersGetters(wxCommandEvent &event)
 		}
 
 		if ( s_dlg->GetFormatText() )
-			DoFormatActiveEditor();
+			DoFormatEditor( &GetCtrl() );
 	}
 }
 
@@ -1288,7 +1302,7 @@ void ContextCpp::OnKeyDown(wxKeyEvent &event)
 		case WXK_UP:
 			ctrl.GetFunctionTip()->SelectPrev(DoGetCalltipParamterIndex());
 			return;
-		
+
 		case WXK_DOWN:
 			ctrl.GetFunctionTip()->SelectNext(DoGetCalltipParamterIndex());
 			return;
@@ -1379,15 +1393,6 @@ void ContextCpp::OnDbgDwellStart(wxScintillaEvent & event)
 {
 	static wxRegEx reCppIndentifier(wxT("[a-zA-Z_][a-zA-Z0-9_]*"));
 
-	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
-	if (dbgr && dbgr->IsRunning() && ManagerST::Get()->DbgCanInteract()) {
-		if ( ManagerST::Get()->GetDisplayVariableDialog()->IsShown() ) {
-			// a 'Quick Show dialog' is already shown!
-			// dont show another tip
-			return;
-		}
-	}
-
 	wxPoint pt;
 	wxString word;
 	pt.x = event.GetX();
@@ -1432,6 +1437,18 @@ void ContextCpp::OnDbgDwellStart(wxScintillaEvent & event)
 		return;
 	}
 
+	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+	if (dbgr && dbgr->IsRunning() && ManagerST::Get()->DbgCanInteract()) {
+		if ( ManagerST::Get()->GetDebuggerTip()->IsShown() && ManagerST::Get()->GetDebuggerTip()->m_expression == word) {
+			// a 'Quick Show dialog' is already shown for this word
+			// dont show another tip
+			return;
+
+		} else {
+			ManagerST::Get()->GetDebuggerTip()->HideDialog();
+
+		}
+	}
 	dbgr->CreateVariableObject( word, DBG_USERR_QUICKWACTH );
 }
 
@@ -1510,7 +1527,7 @@ void ContextCpp::OnMoveImpl(wxCommandEvent &e)
 		if (DoGetFunctionBody(curPos, blockStartPos, blockEndPos, content)) {
 
 			//create the functions body
-			wxString body = TagsManagerST::Get()->FormatFunction(tag, true);
+			wxString body = TagsManagerST::Get()->FormatFunction(tag, FunctionFormat_Impl);
 			//remove the empty content provided by this function
 			body = body.BeforeLast(wxT('{'));
 			body = body.Trim().Trim(false);
@@ -1653,7 +1670,7 @@ void ContextCpp::OnOverrideParentVritualFunctions(wxCommandEvent& e)
 		wxString decl     = dlg.GetDecl();
 		rCtrl.InsertText(rCtrl.GetCurrentPos(), decl);
 		if (dlg.m_checkBoxFormat->IsChecked())
-			DoFormatActiveEditor();
+			DoFormatEditor( &GetCtrl() );
 
 		// Open teh implementation file and format it if needed
 		OpenFileAppendAndFormat(implFile, impl, dlg.m_checkBoxFormat->IsChecked());
@@ -1693,8 +1710,8 @@ void ContextCpp::OnAddMultiImpl(wxCommandEvent &e)
 		TagEntryPtr tag = iter->second;
 		//use normalize function signature rather than the default one
 		//this will ensure that default values are removed
-		tag->SetSignature(TagsManagerST::Get()->NormalizeFunctionSig( tag->GetSignature(), Normalize_Func_Name ));
-		body << TagsManagerST::Get()->FormatFunction(tag, true);
+		tag->SetSignature(TagsManagerST::Get()->NormalizeFunctionSig( tag->GetSignature(), Normalize_Func_Name | Normalize_Func_Reverse_Macro ));
+		body << TagsManagerST::Get()->FormatFunction(tag, FunctionFormat_Impl);
 		body << wxT("\n");
 	}
 
@@ -1723,8 +1740,8 @@ void ContextCpp::OnAddImpl(wxCommandEvent &e)
 	VALIDATE_WORKSPACE();
 
 	//get expression
-	int pos = rCtrl.GetCurrentPos();
-	int word_end = rCtrl.WordEndPosition(pos, true);
+	int pos        = rCtrl.GetCurrentPos();
+	int word_end   = rCtrl.WordEndPosition(pos, true);
 	int word_start = rCtrl.WordStartPosition(pos, true);
 
 	// get the scope
@@ -1733,6 +1750,10 @@ void ContextCpp::OnAddImpl(wxCommandEvent &e)
 	if (word.IsEmpty())
 		return;
 
+	int foundPos(wxNOT_FOUND);
+	if (rCtrl.PreviousChar(word_start, foundPos) == wxT('~'))
+		word.Prepend(wxT("~"));
+
 	std::vector<TagEntryPtr> tags;
 	int line = rCtrl.LineFromPosition(rCtrl.GetCurrentPosition())+1;
 
@@ -1780,9 +1801,9 @@ void ContextCpp::OnAddImpl(wxCommandEvent &e)
 		//create the functions body
 		//replace the function signature with the normalized one, so default values
 		//will not appear in the function implementation
-		wxString newSig = TagsManagerST::Get()->NormalizeFunctionSig( tag->GetSignature(), Normalize_Func_Name );
+		wxString newSig = TagsManagerST::Get()->NormalizeFunctionSig( tag->GetSignature(), Normalize_Func_Name | Normalize_Func_Reverse_Macro);
 		tag->SetSignature( newSig );
-		wxString body = TagsManagerST::Get()->FormatFunction(tag, true);
+		wxString body = TagsManagerST::Get()->FormatFunction(tag, FunctionFormat_Impl);
 
 		wxString targetFile;
 		FindSwappedFile(rCtrl.GetFileName(), targetFile);
@@ -1803,29 +1824,30 @@ void ContextCpp::OnAddImpl(wxCommandEvent &e)
 	}
 }
 
-void ContextCpp::DoFormatActiveEditor()
+void ContextCpp::DoFormatEditor(LEditor *editor)
 {
 	IPlugin *formatter = PluginManager::Get()->GetPlugin(wxT("CodeFormatter"));
-	if (formatter) {
+	if (formatter && editor) {
 		// code formatter is available, format the current source file
 		wxCommandEvent e(wxEVT_COMMAND_MENU_SELECTED, XRCID("format_source"));
+		e.SetString(editor->GetFileName().GetFullPath());
 		Frame::Get()->GetEventHandler()->ProcessEvent(e);
 	}
 }
 
 bool ContextCpp::OpenFileAppendAndFormat(const wxString& fileName, const wxString& text, bool format)
 {
-	OpenFileAndAppend(fileName, text);
-	if (format)
-		DoFormatActiveEditor();
+	LEditor *editor = OpenFileAndAppend(fileName, text);
+	if (format && editor)
+		DoFormatEditor(editor);
 	return true;
 }
 
-bool ContextCpp::OpenFileAndAppend ( const wxString &fileName, const wxString &text )
+LEditor* ContextCpp::OpenFileAndAppend ( const wxString &fileName, const wxString &text )
 {
 	LEditor *editor = Frame::Get()->GetMainBook()->OpenFile(fileName, wxEmptyString, 0);
 	if (!editor)
-		return false;
+		return NULL;
 
 	// if needed, append EOL
 	if (editor->GetText().EndsWith(editor->GetEolString()) == false) {
@@ -1835,7 +1857,7 @@ bool ContextCpp::OpenFileAndAppend ( const wxString &fileName, const wxString &t
 	int lineNum = editor->GetLineCount();
 	editor->GotoLine ( lineNum-1 );
 	editor->AppendText ( text );
-	return true;
+	return editor;
 }
 
 void ContextCpp::OnFileSaved()
@@ -1955,11 +1977,19 @@ void ContextCpp::ApplySettings()
 	LEditor &rCtrl = GetCtrl();
 	rCtrl.SetLexer((int)lexPtr->GetLexerId());
 
-	wxString keyWords = lexPtr->GetKeyWords(0);
+	wxString keyWords     = lexPtr->GetKeyWords(0);
+	wxString doxyKeyWords = lexPtr->GetKeyWords(2);
+
+	// C/C++ keywords
 	keyWords.Replace(wxT("\n"), wxT(" "));
 	keyWords.Replace(wxT("\r"), wxT(" "));
 	rCtrl.SetKeyWords(0, keyWords);
 
+	// Doxygen keywords
+	doxyKeyWords.Replace(wxT("\n"), wxT(" "));
+	doxyKeyWords.Replace(wxT("\r"), wxT(" "));
+	rCtrl.SetKeyWords(2, doxyKeyWords);
+
 	DoApplySettings( lexPtr );
 
 	//create all images used by the cpp context
@@ -2017,7 +2047,21 @@ void ContextCpp::ApplySettings()
 	rCtrl.CmdKeyClear('/', wxSCI_SCMOD_CTRL|wxSCI_SCMOD_SHIFT);
 
 	// update word characters to allow '~' as valid word character
-	rCtrl.SetWordChars(wxT("~_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"));
+	rCtrl.SetWordChars(wxT("_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"));
+
+	// Error
+	wxFont guiFont = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
+	BuildTabSettingsData cmpColoursOptions;
+
+	EditorConfigST::Get()->ReadObject ( wxT ( "build_tab_settings" ), &cmpColoursOptions);
+	rCtrl.StyleSetBackground(eAnnotationStyleError, DrawingUtils::LightColour(cmpColoursOptions.GetErrorColour(), 9.0));
+	rCtrl.StyleSetForeground(eAnnotationStyleError, cmpColoursOptions.GetErrorColour());
+	rCtrl.StyleSetFont(eAnnotationStyleError, guiFont);
+
+	// Warning
+	rCtrl.StyleSetBackground(eAnnotationStyleWarning, DrawingUtils::LightColour(cmpColoursOptions.GetErrorColour(), 9.0));
+	rCtrl.StyleSetForeground(eAnnotationStyleWarning, cmpColoursOptions.GetWarnColour());
+	rCtrl.StyleSetFont(eAnnotationStyleWarning, guiFont);
 }
 
 void ContextCpp::Initialize()
@@ -2375,11 +2419,11 @@ void ContextCpp::OnUserTypedXChars(const wxString &word)
 		std::vector<TagEntryPtr> tags;
 		MakeCppKeywordsTags(word, tags);
 		if ( tags.empty() == false ) {
-			GetCtrl().ShowCompletionBox(tags, 		// list of tags
-			                            word, 		// partial word
-			                            false, 		// dont show full declaration
-			                            true, 		// auto hide if there is no match in the list
-			                            false);		// do not automatically insert word if there is only single choice
+			GetCtrl().ShowCompletionBox(tags,   // list of tags
+			                            word,   // partial word
+			                            false,  // dont show full declaration
+			                            true,   // auto hide if there is no match in the list
+			                            false); // do not automatically insert word if there is only single choice
 		}
 	}
 }
@@ -2535,14 +2579,14 @@ void ContextCpp::DoCodeComplete(long pos)
 		wxString word = rCtrl.GetTextRange(word_start, word_end);
 		rCtrl.GetFunctionTip()->Add( TagsManagerST::Get()->GetFunctionTip(rCtrl.GetFileName(), line, expr, text, word) );
 		rCtrl.GetFunctionTip()->Highlight(DoGetCalltipParamterIndex());
-		
-		// In an ideal world, we would like our tooltip to be placed 
+
+		// In an ideal world, we would like our tooltip to be placed
 		// on top of the caret.
 		wxPoint pt = rCtrl.PointFromPosition(currentPosition);
 		rCtrl.GetFunctionTip()->Activate(pt, rCtrl.GetCurrLineHeight(), rCtrl.StyleGetBackground(wxSCI_C_DEFAULT));
-		
-	} else {
 
+	} else {
+		DoSetProjectPaths();
 		if (TagsManagerST::Get()->AutoCompleteCandidates(rCtrl.GetFileName(), line, expr, text, candidates)) {
 			DisplayCompletionBox(candidates, wxEmptyString, showFullDecl);
 		}
@@ -2573,7 +2617,7 @@ void ContextCpp::GoHyperlink(int start, int end, int type, bool alt)
 		if (type == XRCID("find_tag")) {
 			wxCommandEvent e(wxEVT_COMMAND_MENU_SELECTED,
 			                 alt ? XRCID("find_impl") : XRCID("find_decl"));
-			Frame::Get()->AddPendingEvent(e);
+			Frame::Get()->GetEventHandler()->AddPendingEvent(e);
 		}
 	}
 }
@@ -2587,11 +2631,22 @@ void ContextCpp::DoOpenWorkspaceFile()
 	if (tmpName.Contains(wxT("..")))
 		tmpName = fileName.GetFullName();
 
+#ifdef __WXMSW__
+	// On windows, files are case in-sensitive
+	tmpName.MakeLower();
+#endif
+
 	std::vector<wxFileName> files, files2;
 	TagsManagerST::Get()->GetFiles(fileName.GetFullName(), files);
 	//filter out the all files that does not have an exact match
 	for (size_t i=0; i<files.size(); i++) {
 		wxString curFileName = files.at(i).GetFullPath();
+
+#ifdef __WXMSW__
+		// On windows, files are case in-sensitive
+		curFileName.MakeLower();
+#endif
+
 		curFileName.Replace(wxT("\\"), wxT("/"));
 		if (curFileName.EndsWith(tmpName)) {
 			files2.push_back(files.at(i));
@@ -2780,3 +2835,16 @@ void ContextCpp::SemicolonShift()
 		}
 	}
 }
+void ContextCpp::DoSetProjectPaths()
+{
+	wxArrayString projects;
+	wxArrayString projectPaths;
+	ManagerST::Get()->GetProjectList(projects);
+	for (size_t i=0; i<projects.GetCount(); i++) {
+		ProjectPtr p = ManagerST::Get()->GetProject(projects.Item(i));
+		if (p) {
+			projectPaths.Add(p->GetFileName().GetPath());
+		}
+	}
+	TagsManagerST::Get()->SetProjectPaths( projectPaths );
+}
