@@ -146,7 +146,8 @@ private:
 	int           FindLineToAddInclude();
 	void          MakeCppKeywordsTags(const wxString &word, std::vector<TagEntryPtr> &tags);
 	void          DoOpenWorkspaceFile();
-
+	void          DoSetProjectPaths();
+	
 public:
 	void          DoMakeDoxyCommentString(DoxygenComment &dc);
 
@@ -191,7 +192,7 @@ private:
 	* \param text string text to append
 	* \return true on success, false otherwise
 	*/
-	bool OpenFileAndAppend(const wxString &fileName, const wxString &text);
+	LEditor* OpenFileAndAppend(const wxString &fileName, const wxString &text);
 
 	/**
 	 * @brief open file specified by the 'fileName' parameter and append 'text'
@@ -203,7 +204,11 @@ private:
 	 */
 	bool OpenFileAppendAndFormat(const wxString &fileName, const wxString &text, bool format);
 
-	void DoFormatActiveEditor();
+	/**
+	 * @brief format editor
+	 * @param editor
+	 */
+	void DoFormatEditor(LEditor *editor);
 };
 
 #endif // CONTEXT_CPP_H
