@@ -30,9 +30,11 @@
 #include "sessionmanager.h"
 #include "navbar.h"
 #include "quickfindbar.h"
-#include "custom_notebook.h"
+#include "notebook_ex.h"
 #include "filehistory.h"
+#include "message_pane.h"
 
+class MessagePane;
 class MainBook : public wxPanel
 {
 private:
@@ -40,26 +42,29 @@ private:
 	NavBar       *m_navBar;
 	Notebook     *m_book;
 	QuickFindBar *m_quickFindBar;
-    wxWindow     *m_currentPage;
-
-    std::set<wxWindow*> m_detachedTabs;
+	MessagePane  *m_messagePane;
 
+private:
     void CreateGuiControls();
     void ConnectEvents    ();
 
-    void OnMouseDClick       (wxMouseEvent      &e);
-    void OnFocus             (wxFocusEvent      &e);
-    void OnPaneClosed        (wxAuiManagerEvent &e);
+    void OnMouseDClick       (NotebookEvent     &e);
     void OnPageClosing       (NotebookEvent     &e);
     void OnPageClosed        (NotebookEvent     &e);
+	void OnPageChanged       (NotebookEvent     &e);
+	void OnClosePage         (NotebookEvent     &e);
+
     void OnProjectFileAdded  (wxCommandEvent    &e);
 	void OnProjectFileRemoved(wxCommandEvent    &e);
     void OnWorkspaceLoaded   (wxCommandEvent    &e);
     void OnWorkspaceClosed   (wxCommandEvent    &e);
+	void OnDebugEnded        (wxCommandEvent    &e);
 
 	bool AskUserToSave(LEditor *editor);
 	bool DoSelectPage (wxWindow *win  );
-
+	void DoPositionFindBar(int where);
+	void DoHandleFrameMenu(LEditor *editor);
+	
 public:
 	MainBook(wxWindow *parent);
 	~MainBook();
@@ -68,7 +73,10 @@ public:
 	void GetRecentlyOpenedFiles(wxArrayString &files);
 	FileHistory &GetRecentlyOpenedFilesClass() { return m_recentFiles; }
 
-	void ShowQuickBar (bool s = true)       { m_quickFindBar->Show(s); }
+	void ShowQuickBar (bool s = true)           { m_quickFindBar->Show(s); }
+	void ShowQuickBar (const wxString &findWhat){ m_quickFindBar->Show(findWhat);}
+	void ShowMessage  (const wxString &message, bool showHideButton = true, const wxBitmap &bmp = wxNullBitmap, const ButtonDetails &btn1 = ButtonDetails(), const ButtonDetails &btn2 = ButtonDetails(), const ButtonDetails &btn3 = ButtonDetails());
+
 	void ShowNavBar   (bool s = true);
 	void UpdateNavBar (LEditor *editor);
 	bool IsNavBarShown()                    { return m_navBar->IsShown(); }
@@ -81,7 +89,7 @@ public:
 	LEditor *FindEditor     (const wxString &fileName);
     bool     CloseEditor    (const wxString &fileName) { return ClosePage(FindEditor(fileName)); }
 
-    wxWindow *GetCurrentPage() { return m_currentPage; }
+    wxWindow *GetCurrentPage();
     wxWindow *FindPage      (const wxString &text);
 
     LEditor *NewEditor();
@@ -94,10 +102,6 @@ public:
     bool AddPage   (wxWindow *win, const wxString &text, const wxBitmap &bmp = wxNullBitmap, bool selected = false);
     bool SelectPage(wxWindow *win);
 
-    bool DetachPage(wxWindow *win);
-    bool DockPage  (wxWindow *win);
-    bool IsDetached(wxWindow *win);
-
     bool UserSelectFiles(std::vector<std::pair<wxFileName,bool> > &files, const wxString &title, const wxString &caption,
                          bool cancellable = true);
 
@@ -114,8 +118,7 @@ public:
 	void     SetPageTitle(wxWindow *page, const wxString &name);
 	long     GetBookStyle();
 
-    // TODO: replace these functions with event handlers
-	void ApplySettingsChanges   ();
+    void ApplySettingsChanges   ();
     void UnHighlightAll         ();
     void DelAllBreakpointMarkers();
     void SetViewEOL             (bool visible);
@@ -123,7 +126,7 @@ public:
     void ShowWhitespace         (int ws);
     void UpdateColours          ();
     void UpdateBreakpoints      ();
-	void MarkEditorReadOnly		(LEditor *editor, bool ro);
+	void MarkEditorReadOnly     (LEditor *editor, bool ro);
 };
 
 #endif //MAINBOOK_H
