@@ -122,7 +122,8 @@ static wxArrayString DoGetTemplateTypes(const wxString& tmplDecl)
 	tmpstr.Trim().Trim(false);
 
 	for (size_t i=0; i<tmpstr.Length(); i++) {
-		switch (tmpstr.GetChar(i)) {
+		wxChar ch = tmpstr.GetChar(i);
+		switch (ch) {
 		case wxT(','):
 						if ( depth > 0 ) {
 					type << wxT(",");
@@ -180,7 +181,7 @@ Manager::Manager ( void )
 		, m_useTipWin ( false )
 		, m_tipWinPos ( wxNOT_FOUND )
 		, m_frameLineno ( wxNOT_FOUND )
-		, m_displayVariableDlg (NULL)
+		, m_watchDlg (NULL)
 {
 	m_codeliteLauncher = wxFileName(wxT("codelite_launcher"));
 	Connect(wxEVT_CMD_RESTART_CODELITE,            wxCommandEventHandler(Manager::OnRestart),              NULL, this);
@@ -210,7 +211,6 @@ Manager::~Manager ( void )
 	BuildSettingsConfigST::Free();
 	SearchThreadST::Free();
 	MenuManager::Free();
-	EditorConfigST::Free();
 	EnvironmentConfig::Release();
 
 	if ( m_shellProcess ) {
@@ -257,6 +257,12 @@ void Manager::OpenWorkspace ( const wxString &path )
 		return;
 	}
 
+	// OpenWorkspace returned true, but errMsg is not empty
+	// this could only mean that we removed a fauly project
+	if(errMsg.IsEmpty() == false) {
+		Frame::Get()->GetMainBook()->ShowMessage(errMsg, true, wxXmlResource::Get()->LoadBitmap(wxT("message_pane_warning")));
+	}
+
 	DoSetupWorkspace ( path );
 }
 
@@ -296,7 +302,7 @@ void Manager::DoSetupWorkspace ( const wxString &path )
 	TagsOptionsData tagsopt = TagsManagerST::Get()->GetCtagsOptions();
 	if ( tagsopt.GetFlags() & CC_RETAG_WORKSPACE_ON_STARTUP ) {
 		wxCommandEvent e(wxEVT_COMMAND_MENU_SELECTED, XRCID("retag_workspace"));
-		Frame::Get()->AddPendingEvent(e);
+		Frame::Get()->GetEventHandler()->AddPendingEvent(e);
 	}
 }
 
@@ -469,18 +475,27 @@ void Manager::AddProject ( const wxString & path )
 	SendCmdEvent ( wxEVT_PROJ_ADDED, ( void* ) &projectName );
 }
 
-void Manager::ImportMSVSSolution ( const wxString &path )
+void Manager::ImportMSVSSolution ( const wxString &path, const wxString &defaultCompiler )
 {
 	wxFileName fn ( path );
 	if ( fn.FileExists() == false ) {
 		return;
 	}
+
+	// Show some messages to the user
+	wxBusyCursor busyCursor;
+	wxBusyInfo info(_("Importing MS solution..."), Frame::Get());
+
 	wxString errMsg;
-	VcImporter importer ( path );
+	VcImporter importer ( path, defaultCompiler );
 	if ( importer.Import ( errMsg ) ) {
 		wxString wspfile;
 		wspfile << fn.GetPath() << wxT ( "/" ) << fn.GetName() << wxT ( ".workspace" );
 		OpenWorkspace ( wspfile );
+
+		// Retag workspace
+		wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, XRCID("retag_workspace") );
+		Frame::Get()->GetEventHandler()->AddPendingEvent( event );
 	}
 }
 
@@ -733,6 +748,9 @@ void Manager::RetagWorkspace(bool quickRetag)
 	// Create a parsing request
 	ParseRequest *parsingRequest = new ParseRequest();
 	for (size_t i=0; i<projectFiles.size(); i++) {
+		// filter any non valid coding file
+		if(!TagsManagerST::Get()->IsValidCtagsFile(projectFiles.at(i)))
+			continue;
 		parsingRequest->_workspaceFiles.push_back( projectFiles.at(i).GetFullPath().mb_str(wxConvUTF8).data() );
 	}
 
@@ -1232,31 +1250,94 @@ bool Manager::IsPaneVisible ( const wxString& pane_name )
 	return false;
 }
 
+bool Manager::DoFindDockInfo(const wxString &saved_perspective, const wxString &dock_name, wxString &dock_info)
+{
+	// search for the 'Output View' perspective
+	wxArrayString panes = wxStringTokenize(saved_perspective, wxT("|"), wxTOKEN_STRTOK);
+	for (size_t i=0; i<panes.GetCount(); i++) {
+		if ( panes.Item(i).StartsWith(dock_name) ) {
+			dock_info = panes.Item(i);
+			return true;
+		}
+	}
+	return false;
+}
+
+void Manager::ToggleOutputPane(bool hide)
+{
+	static wxString saved_dock_info;
+	wxAuiManager *aui = &Frame::Get()->GetDockingManager();
+
+	if ( aui ) {
+		wxAuiPaneInfo &pane_info = aui->GetPane(wxT("Output View"));
+		wxString dock_info ( wxString::Format(wxT("dock_size(%d,%d,%d)"), pane_info.dock_direction, pane_info.dock_layer, pane_info.dock_row) );
+		if ( hide ) {
+			if ( pane_info.IsShown() ) {
+				Frame::Get()->Freeze();
+
+				DoFindDockInfo(aui->SavePerspective(), dock_info, saved_dock_info);
+				pane_info.Hide();
+
+				aui->Update();
+
+				Frame::Get()->Thaw();
+			}
+
+
+		} else {
+			// Show it
+			if ( pane_info.IsShown() == false ) {
+				Frame::Get()->Freeze();
+
+				if ( saved_dock_info.IsEmpty() ) {
+
+					pane_info.Show();
+					aui->Update();
+
+				} else {
+					wxString auiPerspective = aui->SavePerspective();
+					if ( auiPerspective.Find(dock_info) == wxNOT_FOUND ) {
+						// the dock_info does not exist
+						auiPerspective << saved_dock_info << wxT("|");
+						aui->LoadPerspective(auiPerspective, false);
+						pane_info.Show();
+						aui->Update();
+					} else {
+						pane_info.Show();
+						aui->Update();
+					}
+				}
+				Frame::Get()->Thaw();
+			}
+		}
+	}
+}
+
+
+
 bool Manager::ShowOutputPane ( wxString focusWin, bool commit )
 {
 	// make the output pane visible
-	bool showedIt ( false );
-
-	wxAuiPaneInfo &info = Frame::Get()->GetDockingManager().GetPane ( wxT ( "Output View" ) );
-	if ( info.IsOk() && !info.IsShown() ) {
-		info.Show();
-		showedIt = true;
-		if ( commit ) {
-			Frame::Get()->GetDockingManager().Update();
-		}
-	}
+	ToggleOutputPane( false );
 
 	// set the selection to focus win
 	OutputPane *pane = Frame::Get()->GetOutputPane();
-	int index = pane->GetNotebook()->GetPageIndex( focusWin );
-	if ( index != wxNOT_FOUND && ( size_t ) index != pane->GetNotebook()->GetSelection() ) {
+	int index(wxNOT_FOUND);
+	for(size_t i=0; i<pane->GetNotebook()->GetPageCount(); i++) {
+		if(pane->GetNotebook()->GetPageText(i) == focusWin) {
+			index = (int)i;
+			break;
+		}
+	}
+
+	if ( index != wxNOT_FOUND && index != pane->GetNotebook()->GetSelection() ) {
 		wxWindow *focus = wxWindow::FindFocus();
 		pane->GetNotebook()->SetSelection ( ( size_t ) index );
 		if (focus) {
 			focus->SetFocus();
 		}
 	}
-	return showedIt;
+	return true;
 }
 
 void Manager::ShowDebuggerPane ( bool show )
@@ -1922,14 +2003,14 @@ void Manager::DbgStart ( long pid )
 	}
 
 	wxString dbgname = dbginfo.path;
-	dbgname = EnvironmentConfig::Instance()->ExpandVariables ( dbgname );
+	dbgname = EnvironmentConfig::Instance()->ExpandVariables ( dbgname, true );
 
 	//set ourselves as the observer for the debugger class
 	dbgr->SetObserver ( this );
-	
+
 	// Set the 'Is remote debugging' flag'
 	dbgr->SetIsRemoteDebugging(bldConf && bldConf->GetIsDbgRemoteTarget() && PID == wxNOT_FOUND);
-	
+
 	// Loop through the open editors and let each editor
 	// a chance to update the debugger manager with any line
 	// changes (i.e. file was edited and breakpoints were moved)
@@ -2037,9 +2118,9 @@ void Manager::DbgStart ( long pid )
 
 void Manager::DbgStop()
 {
-	if ( m_displayVariableDlg ) {
-		m_displayVariableDlg->Destroy();
-		m_displayVariableDlg = NULL;
+	if ( m_watchDlg ) {
+		m_watchDlg->Destroy();
+		m_watchDlg = NULL;
 	}
 
 	// remove all debugger markers
@@ -2121,6 +2202,11 @@ void Manager::DbgUnMarkDebuggerLine()
 
 void Manager::DbgDoSimpleCommand ( int cmd )
 {
+	// Hide tooltip dialog if its ON
+	if(GetDebuggerTip() && GetDebuggerTip()->IsShown()) {
+		GetDebuggerTip()->HideDialog();
+	}
+
 	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
 	if ( dbgr && dbgr->IsRunning() ) {
 		switch ( cmd ) {
@@ -2202,33 +2288,43 @@ void Manager::UpdateGotControl ( DebuggerReasons reason )
 	//put us on top of the z-order window
 	Frame::Get()->Raise();
 	m_dbgCanInteract = true;
-	
+
 	switch ( reason ) {
 	case DBG_RECV_SIGNAL_SIGTRAP:         // DebugBreak()
 	case DBG_RECV_SIGNAL_EXC_BAD_ACCESS:  // SIGSEGV on Mac
 	case DBG_RECV_SIGNAL_SIGABRT:         // assert() ?
 	case DBG_RECV_SIGNAL_SIGSEGV: {       // program received signal sigsegv
-	
+
 		wxString signame = wxT ( "SIGSEGV" );
-		
-		if ( reason == DBG_RECV_SIGNAL_EXC_BAD_ACCESS )
+
+		// show the dialog only if the signal is not sigtrap
+		// since sigtap might be triggered by user inserting a breakpoint
+		// into an already running debug session
+		bool     showDialog(true);
+		if ( reason == DBG_RECV_SIGNAL_EXC_BAD_ACCESS ) {
 			signame = wxT ( "EXC_BAD_ACCESS" );
-			
-		else if ( reason == DBG_RECV_SIGNAL_SIGABRT )
+			showDialog = true;
+
+		} else if ( reason == DBG_RECV_SIGNAL_SIGABRT ) {
 			signame = wxT ( "SIGABRT" );
-			
-		else if ( reason == DBG_RECV_SIGNAL_SIGTRAP )
+			showDialog = true;
+
+		} else if ( reason == DBG_RECV_SIGNAL_SIGTRAP ) {
 			signame = wxT ( "SIGTRAP" );
-		
+			showDialog = false;
+		}
+
 		DebugMessage ( _("Program Received signal ") + signame + _("\n") );
-		wxMessageDialog dlg( Frame::Get(), _("Program Received signal ") + signame + wxT("\n") +
-		                     _("Stack trace is available in the 'Call Stack' tab\n"),
-		                     wxT("CodeLite"), wxICON_ERROR|wxOK );
-		dlg.ShowModal();
+		if(showDialog) {
+			wxMessageDialog dlg( Frame::Get(), _("Program Received signal ") + signame + wxT("\n") +
+								 _("Stack trace is available in the 'Call Stack' tab\n"),
+								 wxT("CodeLite"), wxICON_ERROR|wxOK );
+			dlg.ShowModal();
+		}
 
 		//Print the stack trace
 		wxAuiPaneInfo &info = Frame::Get()->GetDockingManager().GetPane ( wxT("Debugger") );
-		if ( info.IsShown() ) {
+		if ( info.IsShown() && showDialog ) {
 			Frame::Get()->GetDebuggerPane()->SelectTab ( DebuggerPane::FRAMES );
 			UpdateDebuggerPane();
 		}
@@ -2789,21 +2885,21 @@ void Manager::DebuggerUpdate(const DebuggerEvent& event)
 	case DBG_UR_LISTCHILDREN: {
 		IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
 		if ( dbgr && dbgr->IsRunning() && DbgCanInteract() ) {
-			if ( !GetDisplayVariableDialog()->IsShown() ) {
-				GetDisplayVariableDialog()->BuildTree( event.m_varObjChildren, dbgr );
-				GetDisplayVariableDialog()->m_mainVariableObject = event.m_expression;
-				GetDisplayVariableDialog()->ShowDialog( (event.m_userReason == DBG_USERR_WATCHTABLE || event.m_userReason == DBG_USERR_LOCALS) );
+			if ( GetDebuggerTip() && !GetDebuggerTip()->IsShown() ) {
+				GetDebuggerTip()->BuildTree( event.m_varObjChildren, dbgr );
+				GetDebuggerTip()->m_mainVariableObject = event.m_expression;
+				GetDebuggerTip()->ShowDialog( (event.m_userReason == DBG_USERR_WATCHTABLE || event.m_userReason == DBG_USERR_LOCALS) );
 
-			} else {
+			} else if(GetDebuggerTip()) {
 				// The dialog is shown
-				GetDisplayVariableDialog()->AddItems(event.m_expression, event.m_varObjChildren);
+				GetDebuggerTip()->AddItems(event.m_expression, event.m_varObjChildren);
 			}
 		}
 	}
 	break;
 	case DBG_UR_EVALVARIABLEOBJ:
-		if (GetDisplayVariableDialog()->IsShown()) {
-			GetDisplayVariableDialog()->UpdateValue(event.m_expression, event.m_evaluated);
+		if (GetDebuggerTip() && GetDebuggerTip()->IsShown()) {
+			GetDebuggerTip()->UpdateValue(event.m_expression, event.m_evaluated, event.m_displayFormat);
 		}
 		break;
 	case DBG_UR_INVALID:
@@ -2820,7 +2916,7 @@ void Manager::DbgRestoreWatches()
 			DebugMessage(wxT("Restoring watch: ") + m_dbgWatchExpressions.Item(i) + wxT("\n"));
 			wxCommandEvent e(wxEVT_COMMAND_MENU_SELECTED, XRCID("add_watch"));
 			e.SetString(m_dbgWatchExpressions.Item(i));
-			Frame::Get()->GetDebuggerPane()->GetWatchesTable()->AddPendingEvent( e );
+			Frame::Get()->GetDebuggerPane()->GetWatchesTable()->GetEventHandler()->AddPendingEvent( e );
 		}
 	}
 }
@@ -2837,7 +2933,7 @@ void Manager::DoRestartCodeLite()
 	<< wxT("\"");
 
 	wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, XRCID("exit_app"));
-	Frame::Get()->ProcessEvent(event);
+	Frame::Get()->GetEventHandler()->ProcessEvent(event);
 
 	wxExecute(command, wxEXEC_ASYNC|wxEXEC_NOHIDE);
 
@@ -2846,7 +2942,7 @@ void Manager::DoRestartCodeLite()
 	command << wxStandardPaths::Get().GetExecutablePath();
 
 	wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, XRCID("exit_app"));
-	Frame::Get()->AddPendingEvent(event);
+	Frame::Get()->GetEventHandler()->AddPendingEvent(event);
 
 	wxExecute(command, wxEXEC_ASYNC|wxEXEC_NOHIDE);
 #endif
@@ -2863,42 +2959,44 @@ void Manager::OnRestart(wxCommandEvent& event)
 	DoRestartCodeLite();
 }
 
-DisplayVariableDlg* Manager::GetDisplayVariableDialog()
-{
-	if (!m_displayVariableDlg) {
-		m_displayVariableDlg = new DisplayVariableDlg(Frame::Get());
-	}
-	return m_displayVariableDlg;
-}
-
 void Manager::DoShowQuickWatchDialog( const DebuggerEvent &event )
 {
 	/////////////////////////////////////////////
 	// Handle Tooltips
 	/////////////////////////////////////////////
-	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
-	if ( dbgr && dbgr->IsRunning() && DbgCanInteract() ) {
-		GetDisplayVariableDialog()->m_mainVariableObject = event.m_variableObject.gdbId;
-		GetDisplayVariableDialog()->m_variableName       = event.m_expression;
+
+	bool                useDialog   = (event.m_userReason == DBG_USERR_WATCHTABLE || event.m_userReason == DBG_USERR_LOCALS);
+	IDebugger *         dbgr        = DebuggerMgr::Get().GetActiveDebugger();
+	bool                canInteract = (dbgr && dbgr->IsRunning() && DbgCanInteract());
+	DisplayVariableDlg* view        = NULL;
+
+	if(canInteract) {
+
+		// Editor Tooltip
+		view = GetDebuggerTip();
+		view->m_mainVariableObject = event.m_variableObject.gdbId;
+		view->m_variableName       = event.m_expression;
+		view->m_expression         = event.m_expression;
+
 		if ( event.m_evaluated.IsEmpty() == false ) {
-			GetDisplayVariableDialog()->m_variableName << wxT(" = ") << event.m_evaluated;
+			view->m_variableName << wxT(" = ") << event.m_evaluated;
 		}
 
 		if ( event.m_variableObject.typeName.IsEmpty() == false ) {
-			GetDisplayVariableDialog()->m_variableName << wxT(" [") << event.m_variableObject.typeName << wxT("] ");
+			view->m_variableName << wxT(" [") << event.m_variableObject.typeName << wxT("] ");
 		}
 
 		if ( event.m_variableObject.numChilds > 0 ) {
 			// Complex type
 			dbgr->ListChildren(event.m_variableObject.gdbId, event.m_userReason);
 
-		} else {
+		} else  {
 			// Simple type, no need for further calls, show the dialog
-			if ( !GetDisplayVariableDialog()->IsShown() ) {
-				GetDisplayVariableDialog()->BuildTree( event.m_varObjChildren, dbgr );
+			if ( !view->IsShown() ) {
+				view->BuildTree( event.m_varObjChildren, dbgr );
 				// If the reason for showing the dialog was the 'Watches' table being d-clicked,
 				// center the dialog
-				GetDisplayVariableDialog()->ShowDialog( (event.m_userReason == DBG_USERR_WATCHTABLE || event.m_userReason == DBG_USERR_LOCALS) );
+				view->ShowDialog( useDialog );
 			}
 		}
 	}
@@ -2941,6 +3039,8 @@ bool Manager::UpdateParserPaths()
 void Manager::OnIncludeFilesScanDone(wxCommandEvent& event)
 {
 	Frame::Get()->SetStatusMessage(wxT("Retagging..."), 0);
+
+	wxBusyCursor busyCursor;
 	std::set<std::string> *fileSet = (std::set<std::string>*)event.GetClientData();
 //	fprintf(stderr, "fileSet size=%d\n", fileSet->size());
 
@@ -3002,3 +3102,12 @@ void Manager::DoSaveAllFilesBeforeBuild()
 	}
 	SendCmdEvent(wxEVT_FILE_SAVE_BY_BUILD_END);
 }
+
+DisplayVariableDlg* Manager::GetDebuggerTip()
+{
+	if(!m_watchDlg) {
+		m_watchDlg = new DisplayVariableDlg(Frame::Get());
+	}
+	return m_watchDlg;
+}
+
