@@ -39,32 +39,61 @@ void EditHandler::ProcessCommandEvent(wxWindow *owner, wxCommandEvent &event)
 
 	if (event.GetId() == wxID_COPY) {
 		editor->Copy();
+
 	} else if (event.GetId() == wxID_CUT) {
 		editor->Cut();
+
 	} else if (event.GetId() == wxID_PASTE) {
 		editor->Paste();
+
 	} else if (event.GetId() == wxID_UNDO) {
 		editor->Undo();
+
 	} else if (event.GetId() == wxID_REDO) {
 		editor->Redo();
+
 	} else if (event.GetId() == wxID_SELECTALL) {
 		editor->SelectAll();
+
 	} else if (event.GetId() == wxID_DUPLICATE) {
 		editor->SelectionDuplicate();
+
 	} else if (event.GetId() == XRCID("delete_line_end")) {
 		editor->DelLineRight();
+
 	} else if (event.GetId() == XRCID("delete_line_start")) {
 		editor->DelLineLeft();
+
 	} else if (event.GetId() == XRCID("delete_line")) {
 		editor->LineDelete();
+
 	} else if (event.GetId() == XRCID("to_lower")) {
 		editor->LowerCase();
+
 	} else if (event.GetId() == XRCID("to_upper")) {
 		editor->UpperCase();
+
 	} else if (event.GetId() == XRCID("transpose_lines")) {
 		editor->LineTranspose();
+
 	} else if (event.GetId() == wxID_DELETE) {
 		editor->DeleteBack();
+
+	} else if (event.GetId() == XRCID("move_line_down")) {
+
+		int curline  = editor->GetCurrentLine();
+		int lastline = editor->LineFromPosition(editor->GetLength()-1);
+		// Dont transpose if we reached end of the document
+		if(curline == lastline)
+			return;
+
+		editor->LineDown();
+		editor->LineTranspose();
+
+	} else if (event.GetId() == XRCID("move_line_up")) {
+		editor->LineTranspose();
+		editor->LineUp();
+
 	}
 }
 
@@ -131,64 +160,11 @@ void FindReplaceHandler::ProcessCommandEvent(wxWindow *owner, wxCommandEvent &ev
 	}
 
 	if ( event.GetId() == wxID_FIND ) {
-		editor->DoFindAndReplace(false);
+		Frame::Get()->GetMainBook()->ShowQuickBar( editor->GetSelectedText() );
+
 	} else if ( event.GetId() == wxID_REPLACE ) {
 		editor->DoFindAndReplace(true);
-	} else {
-		wxString selWord;
-		if (event.GetId() == XRCID("find_next_at_caret") || event.GetId() == XRCID("find_previous_at_caret")) {
-			wxString selection( editor->GetSelectedText() );
-			if (selection.IsEmpty()) {
-				// select the word
-				selWord = editor->GetWordAtCaret();
-			}
-		}
 
-		FindReplaceDialog *dlg = LEditor::GetFindReplaceDialog();
-		FindReplaceData *data;
-		if (!dlg) {
-			data = &LEditor::GetFindReplaceData();
-		} else {
-			data = &dlg->GetData();
-		}
-
-		// if we have a selected text, use that text instead of one
-		// from the dialog
-		if (editor->GetSelectedText().IsEmpty() == false || selWord.IsEmpty() == false) {
-
-			if(selWord.IsEmpty()) {
-				data->SetFindString(editor->GetSelectedText());
-			}else{
-				// set the word to search from the word under the caret
-				// also set the current search position accordingly
-				if(event.GetId() == XRCID("find_next_at_caret")){
-					// search is going forward, so place the current position at the start of the word
-					editor->SetCurrentPos(editor->WordStartPosition(editor->GetCurrentPos(), true));
-				} else {
-					// search is going backward
-					editor->SetCurrentPos(editor->WordEndPosition(editor->GetCurrentPos(), true));
-				}
-				data->SetFindString(selWord);
-			}
-			if (dlg) {
-				dlg->GetData().SetFindString(editor->GetSelectedText());
-			}
-		}
-
-		if (data->GetFindString().IsEmpty()) {
-			return;
-		}
-
-		if (event.GetId() == XRCID("find_next") || event.GetId() == XRCID("find_next_at_caret")) {
-			// set search direction down
-			data->SetFlags(data->GetFlags() & ~(wxFRD_SEARCHUP));
-			editor->FindNext( *data );
-
-		} else if ( event.GetId() == XRCID("find_previous") || event.GetId() == XRCID("find_previous_at_caret")) {
-			// set search direction up
-			data->SetFlags(data->GetFlags() | wxFRD_SEARCHUP);
-			editor->FindNext( *data );
-		}
 	}
 }
 
