@@ -26,6 +26,7 @@
 #include "editor_config.h"
 #include <wx/statline.h>
 #include "manager.h"
+#include "frame.h"
 #include <wx/textctrl.h>
 #include <wx/wxscintilla.h>
 #include "stringsearcher.h"
@@ -37,7 +38,6 @@ QuickFindBar::QuickFindBar(wxWindow* parent, wxWindowID id)
 		, m_flags(0)
 {
 	Hide();
-	m_closeButton->SetBitmapLabel(wxXmlResource::Get()->LoadBitmap(wxT("page_close16")));
 	DoShowControls();
 
 	GetSizer()->Fit(this);
@@ -48,30 +48,17 @@ QuickFindBar::QuickFindBar(wxWindow* parent, wxWindowID id)
 	wxTheApp->Connect(wxID_COPY,      wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
 	wxTheApp->Connect(wxID_PASTE,     wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
 	wxTheApp->Connect(wxID_SELECTALL, wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
+
+	wxTheApp->Connect(XRCID("find_next"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindNext),          NULL, this);
+	wxTheApp->Connect(XRCID("find_previous"),          wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindPrevious),      NULL, this);
+	wxTheApp->Connect(XRCID("find_next_at_caret"),     wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindNextCaret),     NULL, this);
+	wxTheApp->Connect(XRCID("find_previous_at_caret"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindPreviousCaret), NULL, this);
 }
 
 bool QuickFindBar::Show(bool show)
 {
-	bool res = wxPanel::Show(show);
-	if (res) {
-		GetParent()->GetSizer()->Layout();
-	}
-	
-	DoShowControls();
-	
-	if (!m_sci) {
-		// nothing to do
-	} else if (!show) {
-		m_sci->SetFocus();
-	} else {
-		wxString findWhat = m_sci->GetSelectedText().BeforeFirst(wxT('\n'));
-		if (!findWhat.IsEmpty()) {
-			m_findWhat->SetValue(m_sci->GetSelectedText().BeforeFirst(wxT('\n')));
-		}
-		m_findWhat->SelectAll();
-		m_findWhat->SetFocus();
-	}
-	return res;
+	if (!m_sci && show) return false;
+	return DoShow(show, wxEmptyString);
 }
 
 void QuickFindBar::DoSearch(bool fwd, bool incr)
@@ -91,6 +78,10 @@ void QuickFindBar::DoSearch(bool fwd, bool incr)
 
 	if (!StringFindReplacer::Search(text, offset, find, flags, pos, len)) {
 		// wrap around and try again
+		wxString msg = fwd ? _("Reached end of document, continued from start")
+			               : _("Reached top of document, continued from bottom");
+		Frame::Get()->SetStatusMessage(msg, 0, XRCID("findnext"));
+
 		offset = fwd ? 0 : text.Len()-1;
 		if (!StringFindReplacer::Search(text, offset, find, flags, pos, len)) {
 			m_findWhat->SetBackgroundColour(wxT("PINK"));
@@ -101,10 +92,18 @@ void QuickFindBar::DoSearch(bool fwd, bool incr)
 	m_findWhat->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
 	m_findWhat->Refresh();
 	m_sci->SetSelection(pos, pos+len);
+
+	// Ensure that the found string is visible (e.g. its line isn't folded away)
+	int line = m_sci->LineFromPosition(pos);
+	if ( line >= 0 ) m_sci->EnsureVisible(line);
 }
 
 void QuickFindBar::OnHide(wxCommandEvent &e)
 {
+
+	// Kill any "...continued from start" statusbar message
+	Frame::Get()->SetStatusMessage(wxEmptyString, 0, XRCID("findnext"));
+
 	Show(false);
 	e.Skip();
 }
@@ -131,9 +130,9 @@ void QuickFindBar::OnKeyDown(wxKeyEvent& e)
 {
 	switch (e.GetKeyCode()) {
 	case WXK_ESCAPE: {
-		wxCommandEvent cmd(wxEVT_COMMAND_BUTTON_CLICKED, m_closeButton->GetId());
-		cmd.SetEventObject(m_closeButton);
-		m_closeButton->AddPendingEvent(cmd);
+		wxCommandEvent cmd(wxEVT_COMMAND_TOOL_CLICKED, wxID_HIDE);
+		cmd.SetEventObject(m_toolBar1);
+		m_toolBar1->GetEventHandler()->AddPendingEvent(cmd);
 		break;
 	}
 	default:
@@ -149,11 +148,11 @@ void QuickFindBar::OnUpdateUI(wxUpdateUIEvent &e)
 void QuickFindBar::OnEnter(wxCommandEvent& e)
 {
 	wxUnusedVar(e);
+
 	bool shift = wxGetKeyState(WXK_SHIFT);
-	wxButton *btn = shift ? m_buttonFindPrevious : m_buttonFindNext;
-	wxCommandEvent evt(wxEVT_COMMAND_BUTTON_CLICKED, btn->GetId());
-	evt.SetEventObject(btn);
-	btn->AddPendingEvent(evt);
+	wxCommandEvent evt(wxEVT_COMMAND_TOOL_CLICKED, shift ? wxID_FIND_PREVIOUS : wxID_FIND_NEXT);
+	evt.SetEventObject(this);
+	GetEventHandler()->AddPendingEvent(evt);
 }
 
 void QuickFindBar::OnCopy(wxCommandEvent& e)
@@ -269,7 +268,7 @@ wxTextCtrl* QuickFindBar::GetFocusedControl()
 void QuickFindBar::OnReplaceEnter(wxCommandEvent& e)
 {
 	wxUnusedVar(e);
-	wxCommandEvent evt(wxEVT_COMMAND_BUTTON_CLICKED, m_replaceButton->GetId());
+	wxCommandEvent evt(wxEVT_COMMAND_TOOL_CLICKED, wxID_TOOL_REPLACE);
 	GetEventHandler()->AddPendingEvent(evt);
 }
 
@@ -277,13 +276,13 @@ void QuickFindBar::ShowReplaceControls(bool show)
 {
 	if ( show && !m_replaceWith->IsShown()) {
 		m_replaceWith->Show();
-		m_replaceButton->Show();
+		m_toolBarReplace->Show();
 		m_replaceStaticText->Show();
 		GetSizer()->Layout();
 
 	} else if ( !show && m_replaceWith->IsShown()) {
 		m_replaceWith->Show(false);
-		m_replaceButton->Show(false);
+		m_toolBarReplace->Show(false);
 		m_replaceStaticText->Show(false);
 		GetSizer()->Layout();
 
@@ -322,13 +321,13 @@ void QuickFindBar::OnCheckBoxWord(wxCommandEvent& event)
 
 int QuickFindBar::GetCloseButtonId()
 {
-	return m_closeButton->GetId();
+	return wxID_HIDE;
 }
 
 void QuickFindBar::OnToggleReplaceControls(wxCommandEvent& event)
 {
 	wxUnusedVar(event);
-	long v(m_replaceButton->IsShown() ? 0 : 1);
+	long v(m_replaceWith->IsShown() ? 0 : 1);
 	EditorConfigST::Get()->SaveLongValue(wxT("QuickFindBarShowReplace"), v);
 	DoShowControls();
 }
@@ -339,11 +338,235 @@ void QuickFindBar::DoShowControls()
 	EditorConfigST::Get()->GetLongValue(wxT("QuickFindBarShowReplace"), v);
 	bool canShowToggleReplaceButton = m_sci && !m_sci->GetReadOnly();
 	bool showReplaceControls        = canShowToggleReplaceButton && v;
-	
-	m_showReplaceButton->Show(canShowToggleReplaceButton);
+
 	ShowReplaceControls(showReplaceControls);
-	wxBitmap bmp = showReplaceControls ? wxXmlResource::Get()->LoadBitmap(wxT("expand")) : wxXmlResource::Get()->LoadBitmap(wxT("collapse"));
-	m_showReplaceButton->SetBitmapLabel(bmp);
-	
+	Refresh();
 	GetParent()->GetSizer()->Layout();
 }
+
+bool QuickFindBar::Show(const wxString& findWhat)
+{
+	// Same as Show() but set the 'findWhat' field with findWhat
+	return DoShow(true, findWhat);
+}
+
+bool QuickFindBar::DoShow(bool s, const wxString& findWhat)
+{
+	bool res = wxPanel::Show(s);
+	if (res) {
+		GetParent()->GetSizer()->Layout();
+	}
+
+	DoShowControls();
+
+	if (!m_sci) {
+		// nothing to do
+
+	} else if (!s) {
+		m_sci->SetFocus();
+
+	} else if (findWhat.IsEmpty() == false) {
+		m_findWhat->SetValue(findWhat);
+		m_findWhat->SelectAll();
+		m_findWhat->SetFocus();
+
+	} else {
+		wxString findWhat = m_sci->GetSelectedText().BeforeFirst(wxT('\n'));
+		if (!findWhat.IsEmpty()) {
+			m_findWhat->SetValue(m_sci->GetSelectedText().BeforeFirst(wxT('\n')));
+		}
+		m_findWhat->SelectAll();
+		m_findWhat->SetFocus();
+
+	}
+	return res;
+}
+
+void QuickFindBar::OnFindNext(wxCommandEvent& e)
+{
+	wxUnusedVar(e);
+	if (!m_sci || m_sci->GetLength() == 0)
+		return;
+
+	// Highlighted text takes precedence over the current search string
+	wxString selectedText = m_sci->GetSelectedText();
+	if (selectedText.IsEmpty() == false)
+		m_findWhat->SetValue(selectedText);
+
+	DoSearch(true, false);
+}
+
+void QuickFindBar::OnFindPrevious(wxCommandEvent& e)
+{
+	wxUnusedVar(e);
+	if (!m_sci || m_sci->GetLength() == 0)
+		return;
+
+	// Highlighted text takes precedence over the current search string
+	wxString selectedText = m_sci->GetSelectedText();
+	if (selectedText.IsEmpty() == false)
+		m_findWhat->SetValue(selectedText);
+
+	DoSearch(false, false);
+}
+
+void QuickFindBar::OnFindNextCaret(wxCommandEvent& e)
+{
+	wxUnusedVar(e);
+	if (!m_sci || m_sci->GetLength() == 0)
+		return;
+
+	wxString selection( m_sci->GetSelectedText() );
+	if (selection.IsEmpty()) {
+		// select the word
+		long pos   = m_sci->GetCurrentPos();
+		long start = m_sci->WordStartPosition(pos, true);
+		long end   = m_sci->WordEndPosition(pos, true);
+
+		selection = m_sci->GetTextRange(start, end);
+		if (selection.IsEmpty() == false)
+			m_sci->SetCurrentPos(start);
+	}
+
+	if (selection.IsEmpty())
+		return;
+
+	m_findWhat->SetValue(selection);
+	DoSearch(true, false);
+}
+
+void QuickFindBar::OnFindPreviousCaret(wxCommandEvent& e)
+{
+	wxUnusedVar(e);
+	if (!m_sci || m_sci->GetLength() == 0)
+		return;
+
+	wxString selection( m_sci->GetSelectedText() );
+	if (selection.IsEmpty()) {
+		// select the word
+		long pos   = m_sci->GetCurrentPos();
+		long start = m_sci->WordStartPosition(pos, true);
+		long end   = m_sci->WordEndPosition(pos, true);
+
+		selection = m_sci->GetTextRange(start, end);
+		if (selection.IsEmpty() == false)
+			m_sci->SetCurrentPos(start);
+	}
+
+	if (selection.IsEmpty())
+		return;
+
+	m_findWhat->SetValue(selection);
+	DoSearch(false, false);
+}
+
+
+void QuickFindBar::OnToggleReplaceControlsUI(wxUpdateUIEvent& event)
+{
+	if (ManagerST::Get()->IsShutdownInProgress()) {
+		event.Enable(false);
+
+	} else  if (!m_sci) {
+		event.Enable(false);
+
+	} else {
+		event.Enable(true);
+		event.Check(m_replaceWith->IsShown());
+	}
+}
+
+void QuickFindBar::DoMarkAll()
+{
+	if (!m_sci)
+		return;
+
+	LEditor *editor = dynamic_cast<LEditor*>(m_sci);
+	if (!editor)
+		return;
+
+	wxString findWhat = m_findWhat->GetValue();
+
+	if (findWhat.IsEmpty()) {
+		return;
+	}
+
+	// Save the caret position
+	long savedPos = m_sci->GetCurrentPos();
+	size_t flags  = m_flags;
+
+	int pos(0);
+	int match_len(0);
+
+	// remove reverse search
+	flags &= ~ wxSD_SEARCH_BACKWARD;
+	int offset(0);
+
+	wxString txt = m_sci->GetText();
+	int fixed_offset(0);
+
+	editor->DelAllMarkers();
+
+	// set the active indicator to be 1
+	editor->SetIndicatorCurrent(1);
+
+	while ( StringFindReplacer::Search(txt, offset, findWhat, flags, pos, match_len) ) {
+		editor->MarkerAdd(editor->LineFromPosition(fixed_offset + pos), smt_bookmark);
+
+		// add indicator as well
+		editor->IndicatorFillRange(fixed_offset + pos, match_len);
+		offset = pos + match_len;
+	}
+
+	// Restore the caret
+	editor->SetCurrentPos(savedPos);
+	editor->EnsureCaretVisible();
+}
+
+void QuickFindBar::OnHighlightMatches(wxCommandEvent& event)
+{
+	bool checked;
+#if USE_AUI_TOOLBAR
+	checked = m_toolBar2->GetToolToggled(wxID_HIGHLIGHT_MATCHES);
+#else
+	checked = event.IsChecked();
+#endif
+
+	if(checked) {
+		DoMarkAll();
+	} else {
+		if(m_sci) {
+			LEditor *editor = dynamic_cast<LEditor*>( m_sci );
+			if( editor ) {
+				editor->DelAllMarkers();
+			}
+		}
+	}
+}
+
+void QuickFindBar::OnHighlightMatchesUI(wxUpdateUIEvent& event)
+{
+	if (ManagerST::Get()->IsShutdownInProgress()) {
+		event.Enable(false);
+
+	} else  if (!m_sci) {
+		event.Enable(false);
+
+	} else if ( m_findWhat->GetValue().IsEmpty() ) {
+		event.Enable(false);
+
+	} else {
+		LEditor *editor = dynamic_cast<LEditor*>( m_sci );
+		if( !editor ) {
+			event.Enable(false);
+
+		} else {
+			// Check to see if there are any markers
+			int nLine = editor->LineFromPosition(0);
+			int mask = mmt_bookmarks;
+			int nFoundLine = editor->MarkerNext(nLine + 1, mask);
+
+			event.Enable(true);
+			event.Check(nFoundLine != wxNOT_FOUND);
+		}
+	}
+}
