@@ -23,6 +23,7 @@
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 #include <wx/app.h>
+#include <wx/wupdlock.h>
 #include <wx/xrc/xmlres.h>
 #include "parse_thread.h"
 #include "editor_config.h"
@@ -32,7 +33,7 @@
 #include "manager.h"
 #include "frame.h"
 #include "cl_editor.h"
-#include "custom_notebook.h"
+#include "notebook_ex.h"
 #include "cpp_symbol_tree.h"
 #include "windowstack.h"
 #include "macros.h"
@@ -57,12 +58,7 @@ WorkspacePane::~WorkspacePane()
 {
 }
 
-#define ADD_WORKSPACE_PAGE(win, name) \
-	if( detachedPanes.Index(name) != wxNOT_FOUND ) {\
-		new DockablePane(GetParent(), m_book, win, name, wxNullBitmap, wxSize(200, 200));\
-	} else {\
-		m_book->AddPage(win, name, name, wxNullBitmap, true);\
-	}
+#define IS_DETACHED(name) (detachedPanes.Index(name) != wxNOT_FOUND) ? true : false
 
 void WorkspacePane::CreateGUIControls()
 {
@@ -71,57 +67,90 @@ void WorkspacePane::CreateGUIControls()
 
     // selected configuration:
 
-	mainSizer->Add(new wxStaticText(this, wxID_ANY, wxT("Selected Configuration:")), 0, wxEXPAND| wxTOP|wxLEFT|wxRIGHT, 5);
-
 	wxBoxSizer *hsz = new wxBoxSizer(wxHORIZONTAL);
-	mainSizer->Add(hsz, 0, wxEXPAND|wxALL, 5);
+	mainSizer->Add(hsz, 0, wxEXPAND|wxTOP|wxBOTTOM, 5);
 
 	wxArrayString choices;
 	m_workspaceConfig = new wxChoice(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, choices);
+	m_workspaceConfig->SetToolTip(wxT("Select the workspace build configuration"));
+
 	m_workspaceConfig->Enable(false);
 	m_workspaceConfig->Append(OPEN_CONFIG_MGR_STR);
 	ConnectChoice(m_workspaceConfig, WorkspacePane::OnConfigurationManagerChoice);
-	hsz->Add(m_workspaceConfig, 1, wxEXPAND);
+	hsz->Add(m_workspaceConfig, 1, wxEXPAND| wxALL, 1);
 
-	// add static line separator
-	wxStaticLine *line = new wxStaticLine( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
-	mainSizer->Add(line, 0, wxEXPAND);
+#ifdef __WXMAC__
+	m_workspaceConfig->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+#endif
 
     // add notebook for tabs
-	long bookStyle = wxVB_LEFT|wxVB_FIXED_WIDTH;
-	EditorConfigST::Get()->GetLongValue(wxT("WorkspaceView"), bookStyle);
+	long bookStyle = wxVB_LEFT | wxAUI_NB_SCROLL_BUTTONS;
 	m_book = new Notebook(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, bookStyle);
 
 	// Calculate the widthest tab (the one with the 'Workspcae' label)
 	int xx, yy;
 	wxFont fnt = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
 	wxWindow::GetTextExtent(wxT("Workspace"), &xx, &yy, NULL, NULL, &fnt);
-	m_book->SetFixedTabWidth(xx + 20);
 
-    m_book->SetAuiManager(m_mgr, m_caption);
-	mainSizer->Add(m_book, 1, wxEXPAND | wxALL, 1);
+	mainSizer->Add(m_book, 1, wxEXPAND | wxALL, 0);
 
     // create tabs (possibly detached)
 	DetachedPanesInfo dpi;
 	EditorConfigST::Get()->ReadObject(wxT("DetachedPanesList"), &dpi);
-	wxArrayString detachedPanes = dpi.GetPanes();
 
-	m_workspaceTab = new WorkspaceTab(m_book, wxT("Workspace"));
-	ADD_WORKSPACE_PAGE(m_workspaceTab, m_workspaceTab->GetCaption());
 
-	m_explorer = new FileExplorer(m_book, wxT("Explorer"));
-	ADD_WORKSPACE_PAGE(m_explorer, m_explorer->GetCaption());
+	wxArrayString detachedPanes;
+	detachedPanes = dpi.GetPanes();
 
-	m_winStack = new WindowStack(m_book, wxID_ANY);
-	ADD_WORKSPACE_PAGE(m_winStack, wxT("Outline"));
+	// Add the workspace tab
+	wxString  name;
 
-	m_openWindowsPane = new OpenWindowsPanel(m_book, wxT("Tabs"));
-	ADD_WORKSPACE_PAGE(m_openWindowsPane, m_openWindowsPane->GetCaption());
+	name = wxT("Workspace");
+	if(IS_DETACHED(name)) {
+		DockablePane *cp = new DockablePane(GetParent(), m_book, name, wxNullBitmap, wxSize(200, 200));
+		m_workspaceTab = new WorkspaceTab(cp, name);
+		cp->SetChildNoReparent(m_workspaceTab);
+	} else {
+		m_workspaceTab = new WorkspaceTab(m_book, name);
+		m_book->AddPage(m_workspaceTab, name, true);
+	}
+
+	// Add the explorer tab
+	name = wxT("Explorer");
+	if(IS_DETACHED(name)) {
+		DockablePane *cp = new DockablePane(GetParent(), m_book, name, wxNullBitmap, wxSize(200, 200));
+		m_explorer = new FileExplorer(cp, name);
+		cp->SetChildNoReparent(m_explorer);
+	} else {
+		m_explorer = new FileExplorer(m_book, name);
+		m_book->AddPage(m_explorer, name, false);
+	}
+
+	// Add the Outline tab
+	name = wxT("Outline");
+	if(IS_DETACHED(name)) {
+		DockablePane *cp = new DockablePane(GetParent(), m_book,  name, wxNullBitmap, wxSize(200, 200));
+		m_winStack = new WindowStack(cp);
+		cp->SetChildNoReparent(m_winStack);
+	} else {
+		m_winStack = new WindowStack(m_book);
+		m_book->AddPage(m_winStack, name, false);
+	}
+
+	// Add the Open Windows Panel (Tabs)
+	name = wxT("Tabs");
+	if(IS_DETACHED(name)) {
+		DockablePane *cp = new DockablePane(GetParent(), m_book,  name, wxNullBitmap, wxSize(200, 200));
+		m_openWindowsPane = new OpenWindowsPanel(cp, name);
+		cp->SetChildNoReparent(m_openWindowsPane);
+	} else {
+		m_openWindowsPane = new OpenWindowsPanel(m_book, name);
+		m_book->AddPage(m_openWindowsPane, name, false);
+	}
 
 	if (m_book->GetPageCount() > 0) {
 		m_book->SetSelection((size_t)0);
 	}
-
 	m_mgr->Update();
 }
 
@@ -304,12 +333,12 @@ void WorkspacePane::OnConfigurationManagerChoice(wxCommandEvent &event)
 	BuildMatrixPtr matrix = ManagerST::Get()->GetWorkspaceBuildMatrix();
 	matrix->SetSelectedConfigurationName(selection);
 	ManagerST::Get()->SetWorkspaceBuildMatrix(matrix);
-	
+
 	// Set the focus to the active editor if any
 	LEditor *editor = Frame::Get()->GetMainBook()->GetActiveEditor();
 	if(editor)
 		editor->SetActive();
-	
+
 }
 
 void WorkspacePane::OnConfigurationManager(wxCommandEvent& e)
