@@ -26,10 +26,10 @@
 #define WORKSPACE_PANE_H
 
 #include <wx/filename.h>
+#include "notebook_ex.h"
 #include <wx/panel.h>
 
 // Forward Declarations
-class Notebook;
 class FileViewTree;
 class WindowStack;
 class OpenWindowsPanel;
