@@ -28,6 +28,7 @@
 #include <wx/xrc/xmlres.h>
 #include "dirtraverser.h"
 #include "imanager.h"
+#include "environmentconfig.h"
 #include <wx/dataobj.h>
 #include <wx/stdpaths.h>
 #include "drawingutils.h"
@@ -45,6 +46,7 @@
 #include <wx/clipbrd.h>
 #include "ieditor.h"
 #include <wx/tokenzr.h>
+#include <set>
 
 static wxString DoExpandAllVariables(const wxString &expression, Workspace *workspace, const wxString &projectName, const wxString &confToBuild, const wxString &fileName);
 
@@ -178,7 +180,6 @@ bool RemoveDirectory(const wxString &path)
 	} else {
 		cmd << wxT("\rm -fr ") << wxT("\"") << path << wxT("\"");
 	}
-	wxArrayString output;
 	return wxShell(cmd);
 }
 
@@ -291,6 +292,7 @@ wxString DoExpandAllVariables(const wxString &expression, Workspace *workspace,
 	wxString errMsg;
 	wxString output(expression);
 	if ( workspace ) {
+		output.Replace(wxT("$(WorkspaceName)"), workspace->GetName());
 		ProjectPtr proj = workspace->FindProjectByName(projectName, errMsg);
 		if (proj) {
 			wxString project_name(proj->GetName());
@@ -309,12 +311,12 @@ wxString DoExpandAllVariables(const wxString &expression, Workspace *workspace,
 			}
 			if(output.Find(wxT("$(ProjectFiles)")) != wxNOT_FOUND)
 				output.Replace(wxT("$(ProjectFiles)"),   proj->GetFiles());
-			
+
 			if(output.Find(wxT("$(ProjectFilesAbs)")) != wxNOT_FOUND)
 				output.Replace(wxT("$(ProjectFilesAbs)"),proj->GetFiles(true));
 		}
 	}
-	
+
 	if (fileName.IsEmpty() == false) {
 		wxFileName fn(fileName);
 
@@ -340,9 +342,7 @@ wxString DoExpandAllVariables(const wxString &expression, Workspace *workspace,
 	}
 
 	//call the environment & workspace variables expand function
-	if ( workspace ) {
-		output = workspace->ExpandVariables(output);
-	}
+	output = EnvironmentConfig::Instance()->ExpandVariables(output, true);
 	return output;
 }
 
@@ -626,3 +626,78 @@ void GetProjectTemplateList ( IManager *manager, std::list<ProjectPtr> &list, st
 	}
 }
 
+bool IsCppKeyword(const wxString& word)
+{
+	static std::set<wxString> words;
+
+	if(words.empty()) {
+		words.insert(wxT("auto"));
+		words.insert(wxT("break"));
+		words.insert(wxT("case"));
+		words.insert(wxT("char"));
+		words.insert(wxT("const"));
+		words.insert(wxT("continue"));
+		words.insert(wxT("default"));
+		words.insert(wxT("define"));
+		words.insert(wxT("defined"));
+		words.insert(wxT("do"));
+		words.insert(wxT("double"));
+		words.insert(wxT("elif"));
+		words.insert(wxT("else"));
+		words.insert(wxT("endif"));
+		words.insert(wxT("enum"));
+		words.insert(wxT("error"));
+		words.insert(wxT("extern"));
+		words.insert(wxT("float"));
+		words.insert(wxT("for"));
+		words.insert(wxT("goto"));
+		words.insert(wxT("if"));
+		words.insert(wxT("ifdef"));
+		words.insert(wxT("ifndef"));
+		words.insert(wxT("include"));
+		words.insert(wxT("int"));
+		words.insert(wxT("line"));
+		words.insert(wxT("long"));
+		words.insert(wxT("bool"));
+		words.insert(wxT("pragma"));
+		words.insert(wxT("register"));
+		words.insert(wxT("return"));
+		words.insert(wxT("short"));
+		words.insert(wxT("signed"));
+		words.insert(wxT("sizeof"));
+		words.insert(wxT("static"));
+		words.insert(wxT("struct"));
+		words.insert(wxT("switch"));
+		words.insert(wxT("typedef"));
+		words.insert(wxT("undef"));
+		words.insert(wxT("union"));
+		words.insert(wxT("unsigned"));
+		words.insert(wxT("void"));
+		words.insert(wxT("volatile"));
+		words.insert(wxT("while"));
+		words.insert(wxT("class"));
+		words.insert(wxT("namespace"));
+		words.insert(wxT("delete"));
+		words.insert(wxT("friend"));
+		words.insert(wxT("inline"));
+		words.insert(wxT("new"));
+		words.insert(wxT("operator"));
+		words.insert(wxT("overload"));
+		words.insert(wxT("protected"));
+		words.insert(wxT("private"));
+		words.insert(wxT("public"));
+		words.insert(wxT("this"));
+		words.insert(wxT("virtual"));
+		words.insert(wxT("template"));
+		words.insert(wxT("typename"));
+		words.insert(wxT("dynamic_cast"));
+		words.insert(wxT("static_cast"));
+		words.insert(wxT("const_cast"));
+		words.insert(wxT("reinterpret_cast"));
+		words.insert(wxT("using"));
+		words.insert(wxT("throw"));
+		words.insert(wxT("catch"));
+	}
+
+	return words.find(word) != words.end();
+}
