@@ -60,6 +60,11 @@ void MacrosDlg::Initialize()
 	m_listCtrlMacros->InsertColumn(1, wxT("Description"));
 
 	switch ( m_content ) {
+	case MacrosExternalTools:
+		AddMacro(wxT("$(CurrentSelection)"),      wxT("Expands to the selected text in the active editor"));
+		AddMacro(wxT("$(CurrentSelectionRange)"), wxT("Expands to the selected text range in bytes from beginning of file, eg. 150:200"));
+		AddMacro(wxT("$(ProjectOutputFile)"),     wxT("Expands to the project binary output file"));
+		// fall ...
 	case MacrosProject:
 		AddMacro(wxT("$(ProjectPath)"),           wxT("Expands to project's path"));
 		AddMacro(wxT("$(WorkspacePath)"),         wxT("Expands to workspace's path"));
@@ -94,6 +99,7 @@ void MacrosDlg::Initialize()
 		AddMacro(wxT("$(ArchiveTool)"),            wxT("Expands to the archive tool (e.g. ar) name as set in the Tools tab"));
 		AddMacro(wxT("$(SharedObjectLinkerName)"), wxT("Expands to the shared object linker name as set in the Tools tab"));
 		AddMacro(wxT("$(ObjectSuffix)"),           wxT("Objects suffix (usually set to .o)"));
+		AddMacro(wxT("$(ObjectName)"),             wxT("The object name (without the suffix)"));
 		AddMacro(wxT("$(DependSuffix)"),           wxT("Objects suffix (usually set to .o.d)"));
 		AddMacro(wxT("$(PreprocessSuffix)"),       wxT("Objects suffix (usually set to .o.i)"));
 		AddMacro(wxT("$(IncludeSwitch)"),          wxT("The compiler include switch"));
