@@ -69,11 +69,13 @@ protected:
 	bool           m_hideChangeMarkerMargin;
 	bool           m_hideOutpuPaneOnUserClick;
 	bool           m_hideOutputPaneNotIfDebug;
-	bool           m_showQuickFinder;
+	bool           m_findBarAtBottom;
 	bool           m_TrimLine;
 	bool           m_AppendLF;
 	bool           m_disableSmartIndent;
 	bool           m_disableSemicolonShift;
+	int            m_caretLineAlpha;
+	bool           m_outputPaneDockable;
 
 public:
 	OptionsConfig() {}
@@ -84,6 +86,12 @@ public:
 	// Setters/Getters
 	//-------------------------------------
 
+	void SetOutputPaneDockable(bool outputPaneDockable) {
+		this->m_outputPaneDockable = outputPaneDockable;
+	}
+	bool GetOutputPaneDockable() const {
+		return m_outputPaneDockable;
+	}
 	bool GetDisableSemicolonShift() const {
 		return m_disableSemicolonShift;
 	}
@@ -108,11 +116,11 @@ public:
 	const bool& GetAppendLF() const {
 		return m_AppendLF;
 	}
-	void SetShowQuickFinder(const bool& showQuickFinder) {
-		this->m_showQuickFinder = showQuickFinder;
+	void SetFindBarAtBottom(const bool& findBarAtBottom) {
+		this->m_findBarAtBottom = findBarAtBottom;
 	}
-	const bool& GetShowQuickFinder() const {
-		return m_showQuickFinder;
+	const bool& GetFindBarAtBottom() const {
+		return m_findBarAtBottom;
 	}
 	void SetHideOutpuPaneOnUserClick(const bool& hideOutpuPaneOnUserClick) {
 		this->m_hideOutpuPaneOnUserClick = hideOutpuPaneOnUserClick;
@@ -332,6 +340,12 @@ public:
 	const wxString& GetEolMode() const {
 		return m_eolMode;
 	}
+	void SetCaretLineAlpha(int caretLineAlpha) {
+		this->m_caretLineAlpha = caretLineAlpha;
+	}
+	int GetCaretLineAlpha() const {
+		return m_caretLineAlpha;
+	}
 	/**
 	 * Return an XML representation of this object
 	 * \return XML node
