@@ -36,6 +36,8 @@
 #include "macros.h"
 #include "wx/regex.h"
 #include "globals.h"
+#include "macros.h"
+#include "wx_xml_compatibility.h"
 
 Workspace::Workspace()
 {
@@ -56,11 +58,6 @@ wxString Workspace::GetName() const
 	return wxEmptyString;
 }
 
-wxString Workspace::ExpandVariables(const wxString &expression) const
-{
-	return EnvironmentConfig::Instance()->ExpandVariables(expression);
-}
-
 void Workspace::CloseWorkspace()
 {
 	if (m_doc.IsOk()) {
@@ -81,7 +78,7 @@ bool Workspace::OpenWorkspace(const wxString &fileName, wxString &errMsg)
 	CloseWorkspace();
 	wxFileName workSpaceFile(fileName);
 	if (workSpaceFile.FileExists() == false) {
-		errMsg = wxString::Format(wxT("Workspace file no longer exist: '%s'"), fileName.c_str());
+		errMsg = wxString::Format(wxT("Could not open workspace file: '%s'"), fileName.c_str());
 		return false;
 	}
 
@@ -99,22 +96,27 @@ bool Workspace::OpenWorkspace(const wxString &fileName, wxString &errMsg)
 
 	// Load all projects
 	wxXmlNode *child = m_doc.GetRoot()->GetChildren();
+	std::vector<wxXmlNode*> removedChildren;
+	wxString tmperr;
 	while (child) {
 		if (child->GetName() == wxT("Project")) {
 			wxString projectPath = child->GetPropVal(wxT("Path"), wxEmptyString);
 
 			if ( !DoAddProject(projectPath, errMsg) ) {
-				if (wxMessageBox(wxString::Format(wxT("Error occured while loading project, error was:\n%s\nDo you want to skip it and continue loading the workspace?"), errMsg.c_str()),
-				                 wxT("CodeLite"), wxYES_NO|wxICON_QUESTION|wxCENTER|wxSTAY_ON_TOP, wxTheApp->GetTopWindow()) == wxNO) {
-					return false;
-				} else {
-					wxLogMessage(wxString::Format(wxT("WARNING: Project '%s' was not loaded"), projectPath.c_str()));
-				}
+				tmperr << wxString::Format(wxT("Error occured while loading project: \"%s\"\nCodeLite has removed the faulty project from the workspace\n"), projectPath.c_str());
+				removedChildren.push_back(child);
 			}
 		}
 		child = child->GetNext();
 	}
 
+	// Delete the faulty projects
+	for(size_t i=0; i<removedChildren.size(); i++) {
+		wxXmlNode *ch = removedChildren.at(i);
+		ch->GetParent()->RemoveChild(ch);
+		delete ch;
+	}
+
 	// Load the database
 	wxString dbfile = GetStringProperty(wxT("Database"), errMsg);
 	if ( dbfile.IsEmpty() ) {
@@ -122,6 +124,8 @@ bool Workspace::OpenWorkspace(const wxString &fileName, wxString &errMsg)
 		return false;
 	}
 
+	errMsg = tmperr;
+
 	// the database file names are relative to the workspace,
 	// convert them to absolute path
 	wxFileName fn(dbfile);
@@ -272,10 +276,10 @@ void Workspace::AddProjectToBuildMatrix(ProjectPtr prj)
 		}
 
 		ConfigMappingEntry entry(prj->GetName(), matchConf->GetName());
-		prjList.push_back(entry);
-		(*iter)->SetConfigMappingList(prjList);
-		matrix->SetConfiguration((*iter));
-	}
+			prjList.push_back(entry);
+			(*iter)->SetConfigMappingList(prjList);
+			matrix->SetConfiguration((*iter));
+		}
 
 	// and set the configuration name
 	matrix->SetSelectedConfigurationName(selConfName);
