@@ -23,6 +23,7 @@
 #include <wx/menuitem.h>
 #include <wx/menu.h>
 #include <wx/filedlg.h>
+#include <wx/imaglist.h>
 
 static Subversion2* thePlugin = NULL;
 
@@ -79,27 +80,12 @@ Subversion2::Subversion2(IManager *manager)
 
 Subversion2::~Subversion2()
 {
-	GetManager()->GetTheApp()->Disconnect(XRCID("subversion2_settings"),             wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnSettings),          NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_commit"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnCommit),            NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_update"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnUpdate),            NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_add"),                 wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnAdd),               NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_delete"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnDelete),            NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_revert"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnRevert),            NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_patch"),               wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnPatch),             NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_diff"),                wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnDiff),              NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_log"),                 wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnLog),               NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_blame"),               wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnBlame),             NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_ignore_file"),         wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnIgnoreFile),        NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_ignore_file_pattern"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnIgnoreFilePattern), NULL, this);
-	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_set_as_view"),         wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnSelectAsView),      NULL, this);
 }
 
-wxToolBar *Subversion2::CreateToolBar(wxWindow *parent)
+clToolBar *Subversion2::CreateToolBar(wxWindow *parent)
 {
 	wxUnusedVar(parent);
-	// Create the toolbar to be used by the plugin
-	wxToolBar *tb(NULL);
-	return tb;
+	return NULL;
 }
 
 void Subversion2::CreatePluginMenu(wxMenu *pluginsMenu)
@@ -189,6 +175,23 @@ void Subversion2::UnHookPopupMenu(wxMenu *menu, MenuType type)
 
 void Subversion2::UnPlug()
 {
+	GetManager()->GetTheApp()->Disconnect(XRCID("subversion2_settings"),             wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnSettings),          NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_commit"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnCommit),            NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_update"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnUpdate),            NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_add"),                 wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnAdd),               NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_delete"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnDelete),            NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_revert"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnRevert),            NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_patch"),               wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnPatch),             NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_diff"),                wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnDiff),              NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_log"),                 wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnLog),               NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_blame"),               wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnBlame),             NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_ignore_file"),         wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnIgnoreFile),        NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_ignore_file_pattern"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnIgnoreFilePattern), NULL, this);
+	GetManager()->GetTheApp()->Disconnect(XRCID("svn_explorer_set_as_view"),         wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(Subversion2::OnSelectAsView),      NULL, this);
+	GetManager()->GetTheApp()->Disconnect(wxEVT_GET_ADDITIONAL_COMPILEFLAGS, wxCommandEventHandler(Subversion2::OnGetCompileLine), NULL, this);
+
+	m_subversionView->DisconnectEvents();
+
 	// Remove the tab pined to the workspcae pane
 	size_t index(Notebook::npos);
 	SvnSettingsData ssd = GetSettings();
@@ -200,9 +203,11 @@ void Subversion2::UnPlug()
 	SetSettings(ssd);
 
 	// Remove the tab pined to the output pane
-	index = m_mgr->GetOutputPaneNotebook()->GetPageIndex(m_subversionConsole);
-	if (index != Notebook::npos) {
-		m_mgr->GetOutputPaneNotebook()->RemovePage(index, false);
+	for (size_t i=0; i<m_mgr->GetOutputPaneNotebook()->GetPageCount(); i++) {
+		if (m_subversionConsole == m_mgr->GetOutputPaneNotebook()->GetPage(i)) {
+			m_mgr->GetOutputPaneNotebook()->RemovePage(i);
+			break;
+		}
 	}
 
 	m_subversionView->Destroy();
@@ -213,24 +218,23 @@ void Subversion2::DoInitialize()
 {
 	// create tab (possibly detached)
 	Notebook *book = m_mgr->GetWorkspacePaneNotebook();
-	m_subversionView = new SubversionView(book, this);
 	if( IsSubversionViewDetached() ) {
-		// Make the window child of the main panel (which is the parent of the notebook)
-		new DockablePane(book->GetParent()->GetParent(), book, m_subversionView, svnCONSOLE_TEXT, wxNullBitmap, wxSize(200, 200));
+		// Make the window child of the main panel (which is the grand parent of the notebook)
+		DockablePane *cp = new DockablePane(book->GetParent()->GetParent(), book, svnCONSOLE_TEXT, wxNullBitmap, wxSize(200, 200));
+		m_subversionView = new SubversionView(cp, this);
+		cp->SetChildNoReparent(m_subversionView);
 
 	} else {
+		m_subversionView = new SubversionView(book, this);
 		size_t index = GetSettings().GetSvnTabIndex();
 		if(index == Notebook::npos)
-			book->AddPage(m_subversionView, svnCONSOLE_TEXT, svnCONSOLE_TEXT, wxNullBitmap, true);
+			book->AddPage(m_subversionView, svnCONSOLE_TEXT, false);
 		else
-			book->InsertPage(index, m_subversionView, svnCONSOLE_TEXT, svnCONSOLE_TEXT, wxNullBitmap, true);
+			book->InsertPage(index, m_subversionView, svnCONSOLE_TEXT, false);
 	}
 
-	book = m_mgr->GetOutputPaneNotebook();
-	m_subversionConsole = new SvnConsole(book, this);
-
-	wxBitmap bmp = wxXmlResource::Get()->LoadBitmap(wxT("svn_repo"));
-	book->AddPage(m_subversionConsole, svnCONSOLE_TEXT, svnCONSOLE_TEXT, bmp);
+	m_subversionConsole = new SvnConsole(m_mgr->GetOutputPaneNotebook(), this);
+	m_mgr->GetOutputPaneNotebook()->AddPage(m_subversionConsole, svnCONSOLE_TEXT, false, wxXmlResource::Get()->LoadBitmap(wxT("svn_repo")));
 
 	DoSetSSH();
 	// We need to perform a dummy call to svn so it will create all the default
@@ -292,8 +296,7 @@ void Subversion2::OnAdd(wxCommandEvent& event)
 	if(LoginIfNeeded(event, DoGetFileExplorerItemPath(), loginString) == false) {
 		return;
 	}
-	bool nonInteractive = GetNonInteractiveMode(event);
-	command << GetSvnExeName(nonInteractive) << loginString << wxT(" --recursive add \"") << DoGetFileExplorerItemFullPath() << wxT("\"");
+	command << GetSvnExeName(false) << loginString << wxT(" add \"") << DoGetFileExplorerItemFullPath() << wxT("\"");
 	GetConsole()->Execute(command, DoGetFileExplorerItemPath(), new SvnStatusHandler(this, event.GetId(), this));
 }
 
@@ -330,9 +333,6 @@ void Subversion2::OnRevert(wxCommandEvent& event)
 {
 	wxString command;
 	wxString loginString;
-	if(LoginIfNeeded(event, DoGetFileExplorerItemPath(), loginString) == false) {
-		return;
-	}
 
 	command << GetSvnExeName(false) << loginString << wxT(" revert --recursive \"") << DoGetFileExplorerItemFullPath() << wxT("\"");
 	GetConsole()->Execute(command, DoGetFileExplorerItemPath(), new SvnDefaultCommandHandler(this, event.GetId(), this));
@@ -446,12 +446,12 @@ void Subversion2::RecreateLocalSvnConfigFile()
 	ignorePatterns.Replace(wxT("\n"),   wxT(" "));
 	ignorePatterns.Replace(wxT("\t"),   wxT(" "));
 	ignorePatterns.Replace(wxT("\v"),   wxT(" "));
-	
+
 	wxString diffTool = GetSettings().GetExternalDiffViewer();
 	if(!(GetSettings().GetFlags() & SvnUseExternalDiff)) {
 		diffTool.Empty();
 	}
-	
+
 	wxFFile fp;
 	fp.Open(configFile.c_str(), wxT("w+b"));
 	if(fp.IsOpened()) {
@@ -463,13 +463,13 @@ void Subversion2::RecreateLocalSvnConfigFile()
 		fp.Write(wxT("store-passwords = no\n"));
 		fp.Write(wxT("\n"));
 		fp.Write(wxT("[helpers]\n"));
-		
+
 		if(diffTool.IsEmpty() == false) {
 			fp.Write(wxT("diff-cmd = "));
 			fp.Write(diffTool);
 			fp.Write(wxT("\n"));
 		}
-		
+
 		fp.Close();
 	}
 }
@@ -478,7 +478,7 @@ void Subversion2::DoGetSvnVersion()
 {
 	wxString command;
 	command << GetSvnExeName(false) << wxT(" --version ");
-	m_simpleCommand.Execute(command, wxT(""), new SvnVersionHandler(this, wxNOT_FOUND, NULL));
+	m_simpleCommand.Execute(command, wxT(""), new SvnVersionHandler(this, wxNOT_FOUND, NULL), this);
 }
 
 void Subversion2::Patch(bool dryRun, const wxString &workingDirectory, wxEvtHandler *owner, int id)
@@ -489,9 +489,9 @@ void Subversion2::Patch(bool dryRun, const wxString &workingDirectory, wxEvtHand
 
 
 	wxString patchFile = wxFileSelector(wxT("Select Patch File:"),
-										NULL,
-										NULL,
-										NULL,
+										wxT(""),
+										wxT(""),
+										wxT(""),
 										ALL,
 										0,
 										GetManager()->GetTheApp()->GetTopWindow());
@@ -510,7 +510,7 @@ void Subversion2::Patch(bool dryRun, const wxString &workingDirectory, wxEvtHand
 		} else {
 			handler = new SvnPatchHandler(this, id, owner);
 		}
-		m_simpleCommand.Execute(command, workingDirectory, handler);
+		m_simpleCommand.Execute(command, workingDirectory, handler, this);
 	}
 }
 
@@ -544,10 +544,10 @@ bool Subversion2::LoginIfNeeded(wxCommandEvent& event, const wxString &workingDi
 
 	SvnInfo  svnInfo;
 	wxString repoUrl;
-	
+
 	if(event.GetInt() == LOGIN_REQUIRES_URL) {
 		repoUrl = event.GetString();
-		
+
 	} else {
 		DoGetSvnInfoSync( svnInfo, workingDirectory );
 		repoUrl = svnInfo.m_url;
@@ -689,7 +689,7 @@ void Subversion2::Blame(wxCommandEvent& event, const wxArrayString& files)
 
 	GetConsole()->EnsureVisible();
 	GetConsole()->AppendText(command + wxT("\n"));
-	m_blameCommand.Execute(command, wxT(""), new SvnBlameHandler(this, event.GetId(), this));
+	m_blameCommand.Execute(command, wxT(""), new SvnBlameHandler(this, event.GetId(), this), this);
 }
 
 void Subversion2::OnGetCompileLine(wxCommandEvent& event)
